@@ -21,9 +21,9 @@
 ******************************************************************************/
 
 drd_info_rep::drd_info_rep (string name2):
-  name (name2), info (tag_info ()) {}
+  name (name2), info (tag_info ()), env (UNINIT) {}
 drd_info_rep::drd_info_rep (string name2, drd_info base):
-  name (name2), info (tag_info (), base->info) {}
+  name (name2), info (tag_info (), base->info), env (UNINIT) {}
 drd_info::drd_info (string name):
   rep (tm_new<drd_info_rep> (name)) {}
 drd_info::drd_info (string name, drd_info base):
@@ -304,6 +304,31 @@ drd_info_rep::is_with_like (tree t) {
   return info[L(t)]->pi.with_like && N(t) > 0;
 }
 
+/******************************************************************************
+* The var_type determines whether the tag is a regular macro,
+* or rather an environment variable (which could be a macro).
+******************************************************************************/
+
+void
+drd_info_rep::set_var_type (tree_label l, int vt) {
+  if (info[l]->pi.freeze_with) return;
+  if (!info->contains (l)) info(l)= copy (info[l]);
+  tag_info& ti= info(l);
+  ti->pi.var_type= vt;
+}
+
+int
+drd_info_rep::get_var_type (tree_label l) {
+  return info[l]->pi.var_type;
+}
+
+void
+drd_info_rep::freeze_var_type (tree_label l) {
+  if (!info->contains (l)) info(l)= copy (info[l]);
+  tag_info& ti= info(l);
+  ti->pi.freeze_with= true;
+}
+
 /******************************************************************************
 * Other attributes
 ******************************************************************************/
@@ -333,6 +358,11 @@ drd_info_rep::set_long_name (tree_label l, string val) {
   set_attribute (l, "long-name", val);
 }
 
+void
+drd_info_rep::set_syntax (tree_label l, tree val) {
+  set_attribute (l, "syntax", val);
+}
+
 string
 drd_info_rep::get_name (tree_label l) {
   return as_string (get_attribute (l, "name"));
@@ -345,19 +375,56 @@ drd_info_rep::get_long_name (tree_label l) {
   return as_string (get_attribute (l, "name"));
 }
 
-string
-drd_info_rep::get_class (tree t) {
-  if (is_atomic (t)) {
-    string s= "symbol:" * t->label;
-    if (!existing_tree_label (s)) return "";
-    return as_string (get_attribute (make_tree_label (s), "class"));
+tree
+drd_info_rep::get_syntax (tree_label l) {
+  tree r= get_attribute (l, "syntax");
+  if (r != "") return r;
+  if (env->contains (as_string (l))) return env[as_string (l)];
+  return UNINIT;
+}
+
+static tree
+replace (tree t, hashmap<tree,tree> h) {
+  if (h->contains (t)) return h[t];
+  else if (is_atomic (t)) return t;
+  else {
+    int i, n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= replace (t[i], h);
+    return r;
   }
-  else if (is_func (t, VALUE, 1) && is_atomic (t[0])) {
-    string s= "value:" * t[0]->label;
-    if (!existing_tree_label (s)) return "";
-    return as_string (get_attribute (make_tree_label (s), "class"));
+}
+
+tree
+drd_info_rep::get_syntax (tree t, path p) {
+  if (is_func (t, VALUE, 1) && is_atomic (t[0])) {
+    string s= t[0]->label;
+    if (!existing_tree_label (s)) return UNINIT;
+    return get_syntax (as_tree_label (s));
+  }
+  else if (L(t) < START_EXTENSIONS || is_atomic (t))
+    return UNINIT;
+  else {
+    tree fun= the_drd->get_syntax (L(t));
+    if (fun == UNINIT) return UNINIT;
+    else if (N(t) == 0 && !is_func (fun, MACRO)) return fun;
+    else if (!is_func (fun, MACRO)) return UNINIT;
+    else {
+      int i, n= N(fun)-1;
+      hashmap<tree,tree> tab (UNINIT);
+      for (i=0; i<n; i++) {
+        tree var= tree (ARG, fun[i]);
+        tree val= "";
+        if (i < N(t)) {
+          if (p == path (-1)) val= t[i];
+          else val= tree (QUASI, t[i], (tree) (p * i));
+        }
+        tab (var)= val;
+      }
+      return replace (fun[n], tab);
+    }
   }
-  return as_string (get_attribute (L(t), "class"));
 }
 
 /******************************************************************************
@@ -376,6 +443,7 @@ drd_info_rep::set_type (tree_label l, int nr, int tp) {
 
 int
 drd_info_rep::get_type (tree_label l, int nr) {
+  if (nr >= N(info[l]->ci)) return TYPE_ADHOC;
   return info[l]->ci[nr].type;
 }
 
@@ -383,6 +451,7 @@ void
 drd_info_rep::freeze_type (tree_label l, int nr) {
   if (!info->contains (l)) info(l)= copy (info[l]);
   tag_info  & ti= info(l);
+  if (nr >= N(ti->ci)) return;
   child_info& ci= ti->ci[nr];
   ci.freeze_type= true;
 }
@@ -390,11 +459,12 @@ drd_info_rep::freeze_type (tree_label l, int nr) {
 int
 drd_info_rep::get_type_child (tree t, int i) {
   tag_info ti= info[L(t)];
-  int index= ti->get_index (i, N(t));
   if (is_func (t, EXTERN) && N(t)>0 && is_atomic (t[0])) {
-    ti= info[make_tree_label ("extern:" * t[0]->label)];
-    index= ti->get_index (i-1, N(t));
+    tree_label lab= make_tree_label ("extern:" * t[0]->label);
+    if (info->contains(lab)) { ti= info[lab]; }
+    else { ti = info(EXTERN); info(lab)= ti; }
   }
+  int index= ti->get_index (i, N(t));
   if ((index<0) || (index>=N(ti->ci))) return TYPE_INVALID;
   int r= ti->ci[index].type;
   if (r != TYPE_BINDING) return r;
@@ -418,6 +488,7 @@ drd_info_rep::set_accessible (tree_label l, int nr, int is_accessible) {
 
 int
 drd_info_rep::get_accessible (tree_label l, int nr) {
+  if (nr >= N(info[l]->ci)) return ACCESSIBLE_NEVER;
   return info[l]->ci[nr].accessible;
 }
 
@@ -425,6 +496,7 @@ void
 drd_info_rep::freeze_accessible (tree_label l, int nr) {
   if (!info->contains (l)) info(l)= copy (info[l]);
   tag_info  & ti= info(l);
+  if (nr >= N(ti->ci)) return;
   child_info& ci= ti->ci[nr];
   ci.freeze_accessible= true;
 }
@@ -451,11 +523,12 @@ bool
 drd_info_rep::is_accessible_child (tree t, int i) {
   //cout << "l= " << as_string (L(t)) << "\n";
   tag_info ti= info[L(t)];
-  int index= ti->get_index (i, N(t));
   if (is_func (t, EXTERN) && N(t)>0 && is_atomic (t[0])) {
-    ti= info[make_tree_label ("extern:" * t[0]->label)];
-    index= ti->get_index (i-1, N(t));
+    tree_label lab= make_tree_label ("extern:" * t[0]->label);
+    if (info->contains(lab)) { ti= info[lab]; }
+    else { ti = info(EXTERN); info(lab)= ti; }
   }
+  int index= ti->get_index (i, N(t));
   if ((index<0) || (index>=N(ti->ci))) {
     if (get_access_mode () == DRD_ACCESS_SOURCE)
       return !is_atomic (t) && i >= 0 && i < N(t);
@@ -473,6 +546,15 @@ drd_info_rep::is_accessible_child (tree t, int i) {
   return true; // NOT REACHED
 }
 
+bool
+drd_info_rep::is_accessible_path (tree t, path p) {
+  if (is_nil (p)) return true;
+  return
+    is_accessible_child (t, p->item) &&
+    (p->item < N(t)) &&
+    is_accessible_path (t[p->item], p->next);
+}
+
 /******************************************************************************
 * Children's writability
 ******************************************************************************/
@@ -489,6 +571,7 @@ drd_info_rep::set_writability (tree_label l, int nr, int writability) {
 
 int
 drd_info_rep::get_writability (tree_label l, int nr) {
+  if (nr >= N(info[l]->ci)) return WRITABILITY_NORMAL;
   return info[l]->ci[nr].writability;
 }
 
@@ -496,6 +579,7 @@ void
 drd_info_rep::freeze_writability (tree_label l, int nr) {
   if (!info->contains (l)) info(l)= copy (info[l]);
   tag_info  & ti= info(l);
+  if (nr >= N(ti->ci)) return;
   child_info& ci= ti->ci[nr];
   ci.freeze_writability= true;
 }
@@ -503,11 +587,12 @@ drd_info_rep::freeze_writability (tree_label l, int nr) {
 int
 drd_info_rep::get_writability_child (tree t, int i) {
   tag_info ti= info[L(t)];
-  int index= ti->get_index (i, N(t));
   if (is_func (t, EXTERN) && N(t)>0 && is_atomic (t[0])) {
-    ti= info[make_tree_label ("extern:" * t[0]->label)];
-    index= ti->get_index (i-1, N(t));
+    tree_label lab= make_tree_label ("extern:" * t[0]->label);
+    if (info->contains(lab)) { ti= info[lab]; }
+    else { ti = info(EXTERN); info(lab)= ti; }
   }
+  int index= ti->get_index (i, N(t));
   if ((index<0) || (index>=N(ti->ci))) return WRITABILITY_DISABLE;
   return ti->ci[index].writability;
 }
@@ -539,11 +624,12 @@ drd_info_rep::get_child_long_name (tree_label l, int nr) {
 string
 drd_info_rep::get_child_name (tree t, int i) {
   tag_info ti= info[L(t)];
-  int index= ti->get_index (i, N(t));
   if (is_func (t, EXTERN) && N(t)>0 && is_atomic (t[0])) {
-    ti= info[make_tree_label ("extern:" * t[0]->label)];
-    index= ti->get_index (i-1, N(t));
+    tree_label lab= make_tree_label ("extern:" * t[0]->label);
+    if (info->contains(lab)) { ti= info[lab]; }
+    else { ti = info(EXTERN); info(lab)= ti; }
   }
+  int index= ti->get_index (i, N(t));
   if ((index<0) || (index>=N(ti->ci))) return "";
   return get_child_name (L(t), index);
 }
@@ -551,11 +637,12 @@ drd_info_rep::get_child_name (tree t, int i) {
 string
 drd_info_rep::get_child_long_name (tree t, int i) {
   tag_info ti= info[L(t)];
-  int index= ti->get_index (i, N(t));
   if (is_func (t, EXTERN) && N(t)>0 && is_atomic (t[0])) {
-    ti= info[make_tree_label ("extern:" * t[0]->label)];
-    index= ti->get_index (i-1, N(t));
+    tree_label lab= make_tree_label ("extern:" * t[0]->label);
+    if (info->contains(lab)) { ti= info[lab]; }
+    else { ti = info(EXTERN); info(lab)= ti; }
   }
+  int index= ti->get_index (i, N(t));
   if ((index<0) || (index>=N(ti->ci))) return "";
   string r= get_child_long_name (L(t), index);
   if (r != "") return r;
@@ -582,7 +669,7 @@ drd_env_write (tree env, string var, tree val) {
 tree
 drd_env_merge (tree env, tree t) {
   int i, n= N(t);
-  for (i=0; i<n; i+=2)
+  for (i=0; (i+1)<n; i+=2)
     if (is_atomic (t[i]))
       env= drd_env_write (env, t[i]->label, t[i+1]);
   return env;
@@ -613,6 +700,7 @@ drd_info_rep::set_env (tree_label l, int nr, tree env) {
 
 tree
 drd_info_rep::get_env (tree_label l, int nr) {
+  if (nr >= N(info[l]->ci)) return tree (ATTR);
   return drd_decode (info[l]->ci[nr].env);
 }
 
@@ -620,6 +708,7 @@ void
 drd_info_rep::freeze_env (tree_label l, int nr) {
   if (!info->contains (l)) info(l)= copy (info[l]);
   tag_info  & ti= info(l);
+  if (nr >= N(ti->ci)) return;
   child_info& ci= ti->ci[nr];
   ci.freeze_env= true;
 }
@@ -658,6 +747,7 @@ drd_info_rep::get_env_child (tree t, int i, tree env) {
   }
 }
 
+
 tree
 drd_info_rep::get_env_child (tree t, int i, string var, tree val) {
   tree env= get_env_child (t, i, tree (ATTR));
@@ -678,6 +768,11 @@ drd_info_rep::get_env_descendant (tree t, path p, tree env) {
 * Heuristic initialization of DRD
 ******************************************************************************/
 
+void
+drd_info_rep::set_environment (hashmap<string,tree> env2) {
+  env= env2;
+}
+
 tree
 drd_info_rep::arg_access (tree t, tree arg, tree env, int& type) {
   // returns "" if unaccessible and the env if accessible
@@ -752,6 +847,13 @@ rewrite_symbolic_arguments (tree macro, tree& env) {
     }
 }
 
+static bool
+is_length (string s) {
+  int i;
+  for (i=0; (i<N(s)) && ((s[i]<'a') || (s[i]>'z')); i++) {}
+  return is_double (s (0, i)) && is_locase_alpha (s (i, N(s)));
+}
+
 bool
 drd_info_rep::heuristic_with_like (tree t, tree arg) {
   if (arg == "") {
@@ -772,7 +874,17 @@ drd_info_rep::heuristic_init_macro (string var, tree macro) {
   tag_info old_ti= copy (info[l]);
   int i, n= N(macro)-1;
   set_arity (l, n, 0, ARITY_NORMAL, CHILD_DETAILED);
-  set_type (l, get_type (macro[n]));
+  if (n == 0 && is_compound (macro[0], "localize", 1) &&
+      is_atomic (macro[0][0])) {
+    set_type (l, TYPE_STRING);
+    set_var_type (l, VAR_MACRO_PARAMETER);
+  }
+  else if (n == 0 && is_atomic (macro[0]) &&
+           is_length (macro[0]->label)) {
+    set_type (l, TYPE_LENGTH);
+    set_var_type (l, VAR_MACRO_PARAMETER);
+  }
+  else set_type (l, get_type (macro[n]));
   set_with_like (l, heuristic_with_like (macro, ""));
   //if (heuristic_with_like (macro, ""))
   //cout << "With-like: " << var << LF;
@@ -786,7 +898,7 @@ drd_info_rep::heuristic_init_macro (string var, tree macro) {
     //if (var == "section" || var == "section-title")
     //cout << var << " -> " << env << ", " << macro << "\n";
     //if (var == "math")
-    //cout << var << ", " << i << " -> " << env << ", " << macro << "\n";
+    //cout << var << ", " << i << " -> " << type << ", " << env << ", " << macro << "\n";
     set_type (l, i, type);
     if (env != "") {
       //if (var == "eqnarray*")
@@ -841,9 +953,40 @@ drd_info_rep::heuristic_init_xmacro (string var, tree xmacro) {
   return (old_ti != info[l]);
 }
 
+bool
+drd_info_rep::heuristic_init_parameter (string var, string val) {
+  tree_label l = make_tree_label (var);
+  tag_info old_ti= copy (info[l]);
+  set_arity (l, 0, 0, ARITY_NORMAL, CHILD_UNIFORM);
+  set_var_type (l, VAR_PARAMETER);
+  if (ends (var, "-color")) set_type (l, TYPE_COLOR);
+  else if (ends (var, "-length")) set_type (l, TYPE_LENGTH);
+  else if (ends (var, "-width")) set_type (l, TYPE_LENGTH);
+  else if (val == "true" || val == "false") set_type (l, TYPE_BOOLEAN);
+  else if (is_int (val)) set_type (l, TYPE_INTEGER);
+  else if (is_double (val)) set_type (l, TYPE_NUMERIC);
+  else if (is_length (val)) set_type (l, TYPE_LENGTH);
+  else set_type (l, TYPE_STRING);
+  return (old_ti != info[l]);
+}
+
+bool
+drd_info_rep::heuristic_init_parameter (string var, tree val) {
+  tree_label l = make_tree_label (var);
+  tag_info old_ti= copy (info[l]);
+  set_arity (l, 0, 0, ARITY_NORMAL, CHILD_UNIFORM);
+  set_var_type (l, VAR_PARAMETER);
+  if (ends (var, "-color")) set_type (l, TYPE_COLOR);
+  else if (ends (var, "-length")) set_type (l, TYPE_LENGTH);
+  else if (ends (var, "-width")) set_type (l, TYPE_LENGTH);
+  set_type (l, get_type (val));
+  return (old_ti != info[l]);
+}
+
 void
-drd_info_rep::heuristic_init (hashmap<string,tree> env) {
+drd_info_rep::heuristic_init (hashmap<string,tree> env2) {
   // time_t tt= texmacs_time ();
+  set_environment (env2);
   bool flag= true;
   int round= 0;
   while (flag) {
@@ -853,10 +996,14 @@ drd_info_rep::heuristic_init (hashmap<string,tree> env) {
     while (it->busy()) {
       string var= it->next();
       tree   val= env[var];
-      if (is_func (val, MACRO))
+      if (is_atomic (val))
+        flag= heuristic_init_parameter (var, val->label) | flag;
+      else if (is_func (val, MACRO))
 	flag= heuristic_init_macro (var, val) | flag;
-      if (is_func (val, XMACRO))
+      else if (is_func (val, XMACRO))
 	flag= heuristic_init_xmacro (var, val) | flag;
+      else
+        flag= heuristic_init_parameter (var, val) | flag;
     }
     if ((round++) == 10) {
       cout << "TeXmacs] Warning: bad heuristic drd convergence\n";
