@@ -30,6 +30,9 @@ extern string TABULAR;
 extern string THE_LABEL;
 extern string THE_TAGS;
 extern string THE_MODULES;
+extern string GLOBAL_TITLE;
+extern string GLOBAL_AUTHOR;
+extern string GLOBAL_SUBJECT;
 
 extern string FONT;
 extern string FONT_FAMILY;
@@ -43,6 +46,7 @@ extern string OPACITY;
 extern string BG_COLOR;
 extern string LOCUS_COLOR;
 extern string VISITED_COLOR;
+extern string NO_PATTERNS;
 extern string LANGUAGE;
 extern string ATOM_DECORATIONS;
 extern string LINE_DECORATIONS;
@@ -73,6 +77,9 @@ extern string PROG_SESSION;
 extern string PAR_MODE;
 extern string PAR_FLEXIBILITY;
 extern string PAR_HYPHEN;
+extern string PAR_SPACING;
+extern string PAR_KERNING_STRETCH;
+extern string PAR_KERNING_MARGIN;
 extern string PAR_WIDTH;
 extern string PAR_LEFT;
 extern string PAR_RIGHT;
@@ -244,7 +251,9 @@ extern string CANVAS_BAR_WIDTH;
 extern string CANVAS_BAR_PADDING;
 extern string CANVAS_BAR_COLOR;
 extern string ORNAMENT_SHAPE;
+extern string ORNAMENT_TITLE_STYLE;
 extern string ORNAMENT_BORDER;
+extern string ORNAMENT_SWELL;
 extern string ORNAMENT_HPADDING;
 extern string ORNAMENT_VPADDING;
 extern string ORNAMENT_COLOR;
