@@ -118,18 +118,27 @@ edit_interface_rep::draw_context (renderer ren, rectangle r) {
 }
 
 void
-edit_interface_rep::draw_selection (renderer ren) {
+edit_interface_rep::draw_selection (renderer ren, rectangle r) {
+  rectangles visible (thicken (r, 2 * ren->pixel, 2 * ren->pixel));
   if (!is_nil (locus_rects)) {
     ren->set_pencil (pencil (rgb_color (32, 160, 96), ren->pixel));
     ren->draw_rectangles (locus_rects);
   }
-  if (made_selection && !is_nil (selection_rects)) {
-    ren->set_pencil (pencil (table_selection? rgb_color (192, 0, 255): red,
-                             ren->pixel));
+  for (int i=0; i<N(alt_selection_rects); i++) {
+    ren->set_pencil (pencil (rgb_color (240, 192, 0), ren->pixel));
 #ifdef QTTEXMACS
-    ren->draw_selection (selection_rects);
+    ren->draw_selection (alt_selection_rects[i] & visible);
 #else
-    ren->draw_rectangles (selection_rects);
+    ren->draw_rectangles (alt_selection_rects[i] & visible);
+#endif
+  }
+  if (!is_nil (selection_rects)) {
+    color col= (table_selection? rgb_color (192, 0, 255): red);
+    ren->set_pencil (pencil (col, ren->pixel));
+#ifdef QTTEXMACS
+    ren->draw_selection (selection_rects & visible);
+#else
+    ren->draw_rectangles (selection_rects & visible);
 #endif
   }
 }
@@ -192,7 +201,7 @@ edit_interface_rep::draw_post (renderer win, renderer ren, rectangle r) {
   ren->set_zoom_factor (zoomf);
   draw_context (ren, r);
   draw_env (ren);
-  draw_selection (ren);
+  draw_selection (ren, r);
   draw_graphics (ren);
   draw_cursor (ren); // the text cursor must be drawn over the graphical object
   ren->reset_zoom_factor ();
@@ -304,8 +313,8 @@ void
 edit_interface_rep::handle_repaint (renderer win, SI x1, SI y1, SI x2, SI y2) {
   if (is_nil (eb)) apply_changes ();
   if (env_change != 0) {
-    system_warning ("Invalid situation (" * as_string (env_change) * ")",
-		    "(edit_interface_rep::handle_repaint)");
+    std_warning << "Invalid situation (" << env_change << ")"
+                << " in edit_interface_rep::handle_repaint\n";
     return;
   }
 
