@@ -23,6 +23,17 @@ struct glyph;
 #define FONT_TYPE_QT       2
 #define FONT_TYPE_OTHER    3
 
+#define START_OF_LINE         1
+#define END_OF_LINE           2
+#define PROTRUSION_MASK      60
+#define CJK_PROTRUSION_MASK  28
+#define QUANJIAO              4
+#define BANJIAO               8
+#define HANGMOBANJIAO        12
+#define KAIMING              16
+#define WESTERN_PROTRUSION   32
+#define TABLE_CELL           64
+
 /******************************************************************************
 * The font structure
 ******************************************************************************/
@@ -56,21 +67,33 @@ struct font_rep: rep<font> {
   double   last_zoom;        // last rendered zoom
   font     zoomed_fn;        // zoomed font for last_zoom (or nil)
 
+  hashmap<int,int> protrusion_maps;  // tables for protrusion
+
   font_rep (string name);
   font_rep (string name, font fn);
   void copy_math_pars (font fn);
 
   virtual bool   supports (string c) = 0;
   virtual void   get_extents (string s, metric& ex) = 0;
+  virtual void   get_extents (string s, metric& ex, bool ligf);
+  virtual void   get_extents (string s, metric& ex, SI xk);
   virtual void   get_xpositions (string s, SI* xpos);
+  virtual void   get_xpositions (string s, SI* xpos, bool ligf);
+  virtual void   get_xpositions (string s, SI* xpos, SI xk);
   virtual void   draw_fixed (renderer ren, string s, SI x, SI y) = 0;
+  virtual void   draw_fixed (renderer ren, string s, SI x, SI y, bool ligf);
+  virtual void   draw_fixed (renderer ren, string s, SI x, SI y, SI xk);
   virtual font   magnify (double zoom) = 0;
+  virtual void   draw (renderer ren, string s, SI x, SI y, SI xk, bool ext);
   virtual void   draw (renderer ren, string s, SI x, SI y);
+  virtual void   draw (renderer ren, string s, SI x, SI y, SI xk);
 
   virtual double get_left_slope  (string s);
   virtual double get_right_slope (string s);
   virtual SI     get_left_correction  (string s);
   virtual SI     get_right_correction (string s);
+  virtual SI     get_left_protrusion  (string s, int mode);
+  virtual SI     get_right_protrusion (string s, int mode);
 
   void var_get_extents (string s, metric& ex);
   void var_get_xpositions (string s, SI* xpos);
@@ -139,6 +162,7 @@ array<string> font_database_search (string family, string style);
 array<string> font_database_search (string fam, string var,
                                     string series, string shape);
 array<string> font_database_characteristics (string family, string style);
+tree font_database_substitutions (string family);
 
 // Font selection
 tree array_as_tuple (array<string> a);
@@ -166,6 +190,7 @@ array<string> search_font_exact (array<string> v);
 array<string> search_font_families (array<string> v);
 array<string> search_font_styles (string s, array<string> v);
 array<string> patch_font (array<string> v, array<string> w, bool decode= true);
+array<string> apply_substitutions (array<string> v);
 string main_family (string f);
 
 #endif // defined FONT_H
