@@ -13,6 +13,7 @@
 #include "font.hpp"
 #include "translator.hpp"
 #include "analyze.hpp"
+#include "frame.hpp"
 
 /******************************************************************************
 * The virtual font class
@@ -27,9 +28,17 @@ struct virtual_font_rep: font_rep {
   font_metric  fnm;
   font_glyphs  fng;
   double       unit;
+  hashmap<scheme_tree,metric_struct> trm;
 
   virtual_font_rep (string name, font base, string vname, int size, int dpi);
+  glyph compile_bis (scheme_tree t, metric& ex);
   glyph compile (scheme_tree t, metric& ex);
+  void  get_metric (scheme_tree t, metric& ex);
+  tree  get_tree (string s);
+  void  draw (renderer ren, scheme_tree t, SI x, SI y);
+  void  draw_clipped (renderer ren, scheme_tree t, SI x, SI y,
+                      SI x1, SI y1, SI x2, SI y2);
+  void  draw_transformed (renderer ren, scheme_tree t, SI x, SI y, frame f);
   int   get_char (string s, font_metric& fnm, font_glyphs& fng);
   glyph get_glyph (string s);
 
@@ -45,7 +54,8 @@ virtual_font_rep::virtual_font_rep (
     virt (load_translator (vname)), size (size2), dpi (dpi2),
     last (N(virt->virt_def)),
     fnm (std_font_metric (name, tm_new_array<metric> (last), 0, last-1)),
-    fng (std_font_glyphs (name, tm_new_array<glyph> (last), 0, last-1))
+    fng (std_font_glyphs (name, tm_new_array<glyph> (last), 0, last-1)),
+    trm (metric_struct ())
 {
   copy_math_pars (base_fn);
   unit= ((size*dpi)/72)*PIXEL;
@@ -68,7 +78,7 @@ outer_fit (metric& ex, metric& ey, SI x, SI y) {
 }
 
 glyph
-virtual_font_rep::compile (scheme_tree t, metric& ex) {
+virtual_font_rep::compile_bis (scheme_tree t, metric& ex) {
   //cout << "Compile " << t << "\n";
 
   if (is_atomic (t)) {
@@ -213,11 +223,200 @@ virtual_font_rep::compile (scheme_tree t, metric& ex) {
     return ver_extend (gl, pos, by);
   }
 
-  cerr << "TeXmacs] The defining tree is " << t << "\n";
+  failed_error << "TeXmacs] The defining tree is " << t << "\n";
   FAILED ("invalid virtual character");
   return glyph ();
 }
 
+glyph
+virtual_font_rep::compile (scheme_tree t, metric& ex) {
+  glyph r= compile_bis (t, ex);
+  trm(t)= ex[0];
+  return r;
+}
+
+void
+virtual_font_rep::get_metric (scheme_tree t, metric& ex) {
+  if (trm->contains (t)) ex[0]= trm[t];
+  else (void) compile (t, ex);
+}
+
+/******************************************************************************
+* Direct drawing of virtual fonts using vector graphics
+******************************************************************************/
+
+void
+virtual_font_rep::draw (renderer ren, scheme_tree t, SI x, SI y) {
+  if (is_atomic (t)) {
+    string r= t->label;
+    if (N(r)>1) r= "<" * r * ">";
+    base_fn->draw (ren, r, x, y);
+    return;
+  }
+  
+  if (is_func (t, TUPLE, 3) &&
+      (is_double (t[0])) && (is_double (t[1])))
+    {
+      SI dx= (SI) (as_double (t[0]) * unit);
+      SI dy= (SI) (as_double (t[1]) * unit);
+      draw (ren, t[2], x+dx, y+dy);
+      return;
+    }
+
+  if (is_tuple (t, "join")) {
+    int i, n= N(t);
+    for (i=1; i<n; i++)
+      draw (ren, t[i], x, y);
+    return;
+  }
+
+  if (is_tuple (t, "glue", 2)) {
+    metric ex;
+    get_metric (t[1], ex);
+    SI dx= ex->x2- ((base_fn->wpt*28)>>4);
+    draw (ren, t[1], x, y);
+    draw (ren, t[2], x + dx, y);
+    return;
+  }
+
+  if (is_tuple (t, "glue*", 2)) {
+    metric ex;
+    get_metric (t[1], ex);
+    SI dx= ex->x2;
+    draw (ren, t[1], x, y);
+    draw (ren, t[2], x + dx, y);
+    return;
+  }
+
+  if (is_tuple (t, "add", 2)) {
+    metric ex, ey;
+    get_metric (t[1], ex);
+    get_metric (t[2], ey);
+    SI dx= ((ex->x1+ ex->x2- ey->x1- ey->x2) >> 1);
+    draw (ren, t[1], x, y);
+    draw (ren, t[2], x + dx, y);
+    return;
+  }
+
+  if (is_tuple (t, "enlarge")) {
+    draw (ren, t[1], x, y);
+    return;
+  }
+
+  if (is_tuple (t, "clip")) {
+    metric ex;
+    get_metric (t[1], ex);
+    if (N(t)>2 && t[2]!="*") ex->x3= (SI) (as_double (t[2]) * unit);
+    if (N(t)>3 && t[3]!="*") ex->x4= (SI) (as_double (t[3]) * unit);
+    if (N(t)>4 && t[4]!="*") ex->y3= (SI) (as_double (t[4]) * unit);
+    if (N(t)>5 && t[5]!="*") ex->y4= (SI) (as_double (t[5]) * unit);
+    draw_clipped (ren, t[1], x, y, ex->x3, ex->y3, ex->x4, ex->y4);
+    return;
+  }
+
+  if (is_tuple (t, "hor-flip", 1)) {
+    metric ex;
+    get_metric (t[1], ex);
+    SI ox= x + ((ex->x1 + ex->x2) >> 1);
+    frame f= scaling (point (-1.0, 1.0), point ((double) ox, 0.0));
+    draw_transformed (ren, t[1], (ex->x1 - ex->x2) >> 1, y, f);
+    return;
+  }
+
+  if (is_tuple (t, "ver-flip", 1)) {
+    metric ex;
+    get_metric (t[1], ex);
+    SI oy= y + ((ex->y1 + ex->y2) >> 1);
+    frame f= scaling (point (1.0, -1.0), point (0.0, (double) oy));
+    draw_transformed (ren, t[1], x, (ex->y1 - ex->y2) >> 1, f);
+    return;
+  }
+
+  if (is_tuple (t, "rot-left", 1)) {
+    metric ex;
+    get_metric (t[1], ex);
+    //cout << "left " << (x/PIXEL) << ", " << (y/PIXEL) << "; "
+    //     << (ex->x1/PIXEL) << ", " << (ex->y1/PIXEL) << "; "
+    //     << (ex->x2/PIXEL) << ", " << (ex->y2/PIXEL) << "\n";
+    SI ox= x + ex->x1;
+    SI oy= y + ex->y2;
+    frame f= rotation_2D (point (ox, oy), 1.57079632679);
+    draw_transformed (ren, t[1], x - ex->y2, y + ex->x1, f);
+    return;
+  }
+
+  if (is_tuple (t, "rot-right", 1)) {
+    metric ex;
+    get_metric (t[1], ex);
+    //cout << "right " << (x/PIXEL) << ", " << (y/PIXEL) << "; "
+    //     << (ex->x1/PIXEL) << ", " << (ex->y1/PIXEL) << "; "
+    //     << (ex->x2/PIXEL) << ", " << (ex->y2/PIXEL) << "\n";
+    SI ox= x + ex->x2;
+    SI oy= y + ex->y1;
+    frame f= rotation_2D (point (ox, oy), -1.57079632679);
+    draw_transformed (ren, t[1], x + ex->y1, y - ex->x2, f);
+    return;
+  }
+
+  if (is_tuple (t, "hor-extend", 3) || is_tuple (t, "hor-extend", 4)) {
+    metric ex;
+    get_metric (t[1], ex);
+    SI pos= (SI) (as_double (t[2]) * (ex->x2 - ex->x1));
+    SI add= (SI) (as_double (t[3]) * unit);
+    if (is_tuple (t, "hor-extend", 4))
+      add= (SI) (as_double (t[3]) * as_double (t[4]) * unit);
+    if (add > 0 && ex->x2 > ex->x1) {
+      SI  w = ex->x2 - ex->x1;
+      int n = (int) ((20 * add + w - 1) / w);
+      SI  dx= (add + n - 1) / n;
+      SI  hx= (add + 2*n - 1) / (2*n);
+      for (int i=0; i<n; i++)
+        draw_clipped (ren, t[1], x + hx + i*dx, y,
+                      ex->x3 + pos - hx, ex->y3, ex->x3 + pos + hx, ex->y4);
+    }
+    draw_clipped (ren, t[1], x, y, ex->x3, ex->y3, ex->x3 + pos, ex->y4);
+    draw_clipped (ren, t[1], x + add, y, ex->x3 + pos, ex->y3, ex->x4, ex->y4);
+    return;
+  }
+
+  if (is_tuple (t, "ver-extend", 3) || is_tuple (t, "ver-extend", 4)) {
+    metric ex;
+    get_metric (t[1], ex);
+    SI pos= (SI) (as_double (t[2]) * (ex->y2 - ex->y1));
+    SI add= (SI) (as_double (t[3]) * unit);
+    if (is_tuple (t, "ver-extend", 4))
+      add= (SI) (as_double (t[3]) * as_double (t[4]) * unit);
+    if (add > 0 && ex->y2 > ex->y1) {
+      SI  h = ex->y2 - ex->y1;
+      int n = (int) ((20 * add + h - 1) / h);
+      SI  dy= (add + n - 1) / n;
+      SI  hy= (add + 2*n - 1) / (2*n);
+      for (int i=0; i<n; i++)
+        draw_clipped (ren, t[1], x, y + hy + i*dy - add,
+                      ex->x3, ex->y3 + pos - hy, ex->x4, ex->y3 + pos + hy);
+    }
+    draw_clipped (ren, t[1], x, y - add, ex->x3, ex->y3, ex->x4, ex->y3 + pos);
+    draw_clipped (ren, t[1], x, y, ex->x3, ex->y3 + pos, ex->x4, ex->y4);
+    return;
+  }
+}
+
+void
+virtual_font_rep::draw_clipped (renderer ren, scheme_tree t, SI x, SI y,
+                                SI x1, SI y1, SI x2, SI y2) {
+  ren->clip (x + x1, y + y1, x + x2, y + y2);
+  draw (ren, t, x, y);
+  ren->unclip ();  
+}
+
+void
+virtual_font_rep::draw_transformed (renderer ren, scheme_tree t, SI x, SI y,
+                                    frame f) {
+  ren->set_transformation (f);
+  draw (ren, t, x, y);
+  ren->reset_transformation ();  
+}
+
 /******************************************************************************
 * Getting extents and drawing strings
 ******************************************************************************/
@@ -279,6 +478,19 @@ virtual_font_rep::get_char (string s, font_metric& cfnm, font_glyphs& cfng) {
   }
 }
 
+tree
+virtual_font_rep::get_tree (string s) {
+  int c= ((N(s)==0)? -1: ((QN) s[0]));
+  if ((c<0) || (c>=last)) return "";
+  if (N(s)==1) return virt->virt_def[c];
+  else if (s[0] == '<' && s[N(s)-1] == '>') {
+    if (!virt->dict->contains (s)) return "";
+    int c2= virt->dict [s];
+    return virt->virt_def[c2];
+  }
+  else return subst_sharp (virt->virt_def[c], s(1,N(s)));
+}
+
 bool
 virtual_font_rep::supports (string s) {
   font_metric cfnm;
@@ -307,10 +519,16 @@ virtual_font_rep::get_extents (string s, metric& ex) {
 
 void
 virtual_font_rep::draw_fixed (renderer ren, string s, SI x, SI y) {
-  font_metric cfnm;
-  font_glyphs cfng;
-  int c= get_char (s, cfnm, cfng);
-  if (c != -1) ren->draw (c, cfng, x, y);
+  if (ren->is_screen) {
+    font_metric cfnm;
+    font_glyphs cfng;
+    int c= get_char (s, cfnm, cfng);
+    if (c != -1) ren->draw (c, cfng, x, y);
+  }
+  else {
+    tree t= get_tree (s);
+    if (t != "") draw (ren, t, x, y);
+  }
 }
 
 font
