@@ -10,6 +10,7 @@
 ******************************************************************************/
 
 #include "bibtex_functions.hpp"
+#include "converter.hpp"
 #include "vars.hpp"
 
 /******************************************************************************
@@ -139,7 +140,7 @@ bib_to_latex (string s) {
   while (pos < N(s)) {
     specialsav= special;
     string ch= bib_parse_char (s, pos, depth, special, math);
-    //    cerr << ch << " " << as_string (keepcase) << " " << as_string (depth) << "\n";
+    // convert_warning << ch << " " << as_string (keepcase) << " " << as_string (depth) << "\n";
     if (ch == "$$") r << "$";
     else if (special || math) r << ch;
     else if (bib_is_space (ch)) {
@@ -165,7 +166,7 @@ bib_to_latex (string s) {
   }
   for (int i= keepcase; i>0; i--) r << "}";
   r << "}";
-  //  cerr << r << "\n";
+  // convert_warning << r << "\n";
   return r;
 }
 
@@ -901,7 +902,7 @@ bib_parse_fields (tree& t) {
   int i= 0;
   int nb= bib_get_fields (t, fields);
   array<tree> latex= bib_latex_array (
-      latex_to_tree (parse_latex (fields, false, true, false, false)));
+      latex_to_tree (parse_latex (cork_to_sourcecode (fields), false, false)));
   if (nb == N(latex)) bib_set_fields (t, latex, i);
 }
 
@@ -1041,11 +1042,16 @@ bib_select_entries (tree t, tree bib_t) {
   tree entries (DOCUMENT);
   tree bt= copy (bib_t);
   for (int i= 0; i<N(t); i++)
-    if (bib_is_entry (t[i]))
-      h(t[i][1]->label)= t[i];
+    if (bib_is_entry (t[i])) {
+      if (h->contains (t[i][1]->label))
+        bibtex_warning << "Duplicate entry '" << t[i][1]->label << "'\n";
+      else h(t[i][1]->label)= t[i];
+    }
   for (int i= 0; i < arity (bt); i++) {
     string b= as_string (bt[i]);
-    if (h->contains (b) && !r->contains (b)) {
+    if (!h->contains (b))
+      bibtex_warning << "Missing reference '" << b << "'\n";
+    else if (!r->contains (b)) {
       r->insert (b);
       entries << h[b];
       tree cr= bib_assoc (h[b], string ("crossref"));
