@@ -56,8 +56,8 @@ tt_font_rep::tt_font_rep (string name, string family2, int size2, int dpi2):
     fnm= std_font_metric (res_name, NULL, 0, -1);
     fng= std_font_glyphs (res_name, NULL, 0, -1);
     if (DEBUG_AUTO)
-      cout << "TeXmacs] Font " << family << " " << size
-	   << "pt at " << dpi << " dpi could not be loaded\n";
+      debug_fonts << "Font " << family << " " << size << "pt "
+                  << "at " << dpi << " dpi could not be loaded\n";
     
   }
 
@@ -95,8 +95,8 @@ tt_font_rep::tt_font_rep (string name, string family2, int size2, int dpi2):
 
   // get space length
   get_extents (" ", ex);
-  spc  = space ((3*(ex->x2-ex->x1))>>2, ex->x2-ex->x1, (ex->x2-ex->x1)<<1);
-  extra= spc;
+  spc  = space ((3*(ex->x2-ex->x1))>>2, ex->x2-ex->x1, (3*(ex->x2-ex->x1))>>1);
+  extra= spc/2;
   sep  = wfn/10;
 
   // get_italic space
@@ -213,7 +213,7 @@ tt_font (string family, int size, int dpi) {
 font
 tt_font (string family, int size, int dpi) {
   string name= "tt:" * family * as_string (size) * "@" * as_string(dpi);
-  cerr << "\n\nFont name= " << name << "\n";
+  failed_error << "Font name= " << name << "\n";
   FAILED ("true type support was disabled");
   return font ();
 }
