@@ -9,14 +9,19 @@
 * in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
-#include "QTMMenuHelper.hpp"
-#include "QTMGuiHelper.hpp"
-#include "QTMStyle.hpp"
+#include "analyze.hpp"
+
 #include "qt_gui.hpp"
 #include "qt_utilities.hpp"
 #include "qt_window_widget.hpp"
-#include "qt_ui_element.hpp"  // qt_choice_command_rep
-#include "analyze.hpp"
+#include "qt_ui_element.hpp"    // qt_choice_command_rep
+#include "qt_picture.hpp"       // xpm_image
+#include "qt_tm_widget.hpp"     // tweak_iconbar_size
+#include "QTMMenuHelper.hpp"
+#include "QTMGuiHelper.hpp"
+#include "QTMStyle.hpp"
+#include "QTMApplication.hpp"
+#include "QTMTreeModel.hpp"
 
 #include <QToolTip>
 #include <QCompleter>
@@ -29,9 +34,16 @@
 /*! Queues the object's command into the main queue. */
 void 
 QTMCommand::apply()  {
-  if (DEBUG_QT)
-    cout << "QTMCommand::apply() (delayed): " << cmd << "\n";
-  if (!is_nil(cmd)) { the_gui->process_command(cmd); }
+  if (!is_nil (cmd)) {
+    the_gui->process_command (cmd);
+    if (DEBUG_QT) {
+      debug_qt << "QTMCommand::apply() (delayed)\n";
+      /* FIXME: this sometimes crashes:
+      cmd->print(debug_qt);
+      debug_qt << "\n";
+       */
+    }
+  }
 }
 
 
@@ -39,14 +51,16 @@ QTMCommand::apply()  {
  * QTMAction
  ******************************************************************************/
 
-QTMAction::QTMAction(QObject *parent) : QAction(parent) { 
-  QObject::connect(the_gui->gui_helper, SIGNAL(refresh()), this, SLOT(doRefresh()));
-  _timer = new QTimer(this);
-  QObject::connect(_timer, SIGNAL(timeout()), this, SLOT(doShowToolTip()));
+QTMAction::QTMAction (QObject *parent) : QAction (parent) {
+  QObject::connect (the_gui->gui_helper, SIGNAL (refresh()),
+                    this,                  SLOT (doRefresh()));
+  _timer = new QTimer (this);
+  QObject::connect (_timer, SIGNAL (timeout()),
+                    this,     SLOT (doShowToolTip()));
 }
 
 QTMAction::~QTMAction() { 
-  if (menu() && !(menu()->parent())) delete menu(); 
+  if (menu() && !menu()->parent()) delete menu();
 }
 
 void
@@ -68,7 +82,7 @@ QTMAction::doRefresh() {
 void
 QTMAction::showToolTip()
 {
-  _timer->start(500);   // Restarts the timer if already running
+  _timer->start (500);   // Restarts the timer if already running
   _pos = QCursor::pos();
 }
 
@@ -88,8 +102,8 @@ void
 QTMAction::doShowToolTip() {
   static int step = QApplication::font().pointSize();
   _timer->stop();
-  if((QCursor::pos() - _pos).manhattanLength() < step)  // Hideous HACK
-    QToolTip::showText(QCursor::pos(), toolTip());
+  if ((QCursor::pos() - _pos).manhattanLength() < step)  // Hideous HACK
+    QToolTip::showText (QCursor::pos(), toolTip());
   else
     QToolTip::hideText();
 }
@@ -102,13 +116,14 @@ QTMAction::doShowToolTip() {
 
 QTMWidgetAction::QTMWidgetAction (widget _wid, QObject *parent)
 : QWidgetAction (parent), wid (_wid) {
-  QObject::connect (the_gui->gui_helper, SIGNAL(refresh()), this, SLOT(doRefresh()));
+  QObject::connect (the_gui->gui_helper, SIGNAL (refresh()),
+                    this,                  SLOT (doRefresh()));
 }
 
 QWidget *
 QTMWidgetAction::createWidget (QWidget * parent) {
-  QWidget* qw = concrete(wid)->as_qwidget();
-  qw->setParent(parent);
+  QWidget* qw = concrete (wid)->as_qwidget();
+  qw->setParent (parent);
   return qw;
 }
 
@@ -117,15 +132,14 @@ QTMWidgetAction::createWidget (QWidget * parent) {
  * QTMTileAction
  ******************************************************************************/
 
-QTMTileAction::QTMTileAction (QWidget* parent, array<widget>& arr, int _cols)
-: QWidgetAction (parent), cols (_cols)
-{
-  actions.reserve(N(arr));
-  for(int i = 0; i < N(arr); i++) {
-    if (is_nil(arr[i])) break;
-    QAction *act = concrete(arr[i])->as_qaction();
-    act->setParent(this);
-    actions.append(act);
+QTMTileAction::QTMTileAction (array<widget>& arr, int _cols, QObject* parent)
+: QWidgetAction (parent), cols (_cols) {
+  actions.reserve (N (arr));
+  for (int i = 0; i < N (arr); i++) {
+    if (is_nil (arr[i])) break;
+    QAction* act = concrete (arr[i])->as_qaction();
+    act->setParent (this);
+    actions.append (act);
   };
 }
 
@@ -137,25 +151,25 @@ QTMTileAction::QTMTileAction (QWidget* parent, array<widget>& arr, int _cols)
  the widget so it results in a waste of time.
  */
 QWidget*
-QTMTileAction::createWidget(QWidget* parent)
+QTMTileAction::createWidget (QWidget* parent)
 {
   if (DEBUG_QT_WIDGETS)
-    cout << "QTMTileAction::createWidget\n";
+    debug_widgets << "QTMTileAction::createWidget\n";
   QWidget* wid= new QTMMenuWidget (parent);
   QGridLayout* l= new QGridLayout (wid);
-    // wid->setAutoFillBackground(true);
-    // wid->setBackgroundRole(QPalette::Base);
+    // wid->setAutoFillBackground (true);
+    // wid->setBackgroundRole (QPalette::Base);
   wid->setLayout (l);
   l->setSizeConstraint (QLayout::SetFixedSize);
   l->setHorizontalSpacing (2);
   l->setVerticalSpacing (2);
   l->setContentsMargins (4, 0, 4, 0);
-  int row= 0, col= 0;
-  for (int i=0; i < actions.count(); i++) {
-    QAction* sa= actions[i];
+  int row = 0, col = 0;
+  for (int    i = 0; i < actions.count(); i++) {
+    QAction* sa = actions[i];
     QToolButton* tb= new QTMMenuButton (wid);
     tb->setDefaultAction (sa);
-    QObject::connect(tb, SIGNAL(released()), this, SLOT(trigger()));
+    QObject::connect (tb, SIGNAL (released()), this, SLOT (trigger()));
       //  tb->setStyle (qtmstyle ());
     l->addWidget (tb, row, col);
     col++;
@@ -169,15 +183,15 @@ QTMTileAction::createWidget(QWidget* parent)
  * QTMMinibarAction
  ******************************************************************************/
 
-QTMMinibarAction::QTMMinibarAction (QWidget* parent, array<widget>& arr)
+QTMMinibarAction::QTMMinibarAction (array<widget>& arr, QObject* parent)
 : QWidgetAction (parent)
 {
-  actions.reserve(N(arr));
-  for(int i = 0; i < N(arr); i++) {
-    if (is_nil(arr[i])) break;
-    QAction *act = concrete(arr[i])->as_qaction();
-    act->setParent(this);
-    actions.append(act);
+  actions.reserve (N (arr));
+  for (int i = 0; i < N (arr); i++) {
+    if (is_nil (arr[i])) break;
+    QAction *act = concrete (arr[i])->as_qaction();
+    act->setParent (this);
+    actions.append (act);
   };
 }
 
@@ -189,7 +203,11 @@ QTMMinibarAction::QTMMinibarAction (QWidget* parent, array<widget>& arr)
  */
 QWidget*
 QTMMinibarAction::createWidget (QWidget* parent) {
-  if (DEBUG_QT_WIDGETS) cout << "QTMMinibarAction::createWidget\n";
+  static QImage* pxm = xpm_image ("tm_add.xpm"); // See qt_tm_widget.cpp 
+  QSize sz = pxm ? pxm->size() : QSize (16, 16);
+  qt_tm_widget_rep::tweak_iconbar_size (sz);
+  
+  if (DEBUG_QT_WIDGETS) debug_widgets << "QTMMinibarAction::createWidget\n";
   QWidget* wid= new QWidget (parent);
   QBoxLayout* l= new QBoxLayout (QBoxLayout::LeftToRight, wid);
   wid->setLayout (l);
@@ -199,62 +217,155 @@ QTMMinibarAction::createWidget (QWidget* parent) {
   for (int i=0; i < actions.count(); i++) {
     QAction* sa= actions[i];
     if (QWidgetAction * wa = qobject_cast<QWidgetAction*> (sa)) {
-      QWidget *w = wa->requestWidget(wid);
+      QWidget *w = wa->requestWidget (wid);
       l->addWidget(w);
     } else if (sa->text().isNull() && sa->icon().isNull()) {
       l->addSpacing(8);
     } else {
-      QToolButton *tb = new QToolButton(wid);
+      QToolButton *tb = new QToolButton (wid);
       
         //HACK: texmacs does not use the checked state of the action
         // if the action is checkable then it means that it should be checked
-      sa->setChecked(sa->isCheckable());
+      sa->setChecked (sa->isCheckable());
       
       tb->setDefaultAction (sa);
       tb->setAutoRaise (true);
       tb->setPopupMode (QToolButton::InstantPopup);
       tb->setStyle (qtmstyle());
-        //  tb->setIconSize(QSize(12,12));
+      tb->setIconSize (sz);
       QFont f = tb->font();
       int fs = as_int (get_preference ("gui:mini-fontsize", QTM_MINI_FONTSIZE));
       f.setPixelSize (fs > 0 ? fs : QTM_MINI_FONTSIZE);
       tb->setFont(f);
-      l->addWidget(tb);
+      l->addWidget (tb);
     }
   }
   return wid;
 }
 
+/******************************************************************************
+ * QTMMenuButton
+ ******************************************************************************/
+
+QTMMenuButton::QTMMenuButton (QWidget* parent) : QToolButton (parent) {
+  QTMAuxMenu m;
+  m.myInitStyleOption (&option);
+  setAttribute (Qt::WA_Hover);
+}
+
+void
+QTMMenuButton::mousePressEvent (QMouseEvent* e) {
+    // this one triggers the action and toggles the button
+  QToolButton::mousePressEvent (e);
+    // this one forwards the event to the parent
+    // (which eventually is the menu)
+  QWidget::mousePressEvent (e);
+}
+
+void
+QTMMenuButton::mouseReleaseEvent (QMouseEvent* e) {
+    // this one triggers the action and untoggles the button
+  QToolButton::mouseReleaseEvent (e);
+    // this one forwards the event to the parent
+    // (which eventually is the menu which then closes itself)
+  QWidget::mouseReleaseEvent (e);
+}
+
+void
+QTMMenuButton::paintEvent (QPaintEvent* e) {
+  (void) e;
+  QPainter p (this);
+  
+    // initialize the options
+  QStyleOptionToolButton opt;
+  initStyleOption (&opt);
+  QRect r = rect();
+  option.rect = r;
+  option.state = QStyle::State_Enabled | (opt.state & QStyle::State_MouseOver
+                                          ? QStyle::State_Selected
+                                          : QStyle::State_None);
+    // draw the control background as a menu item
+  style()->drawControl (QStyle::CE_MenuItem, &option, &p, this);
+    // draw the icon with a bit of inset.
+  r.adjust (2, 2, -2, -2);
+  defaultAction()->icon().paint (&p, r);
+}
+
+/******************************************************************************
+ * QTMMenuWidget
+ ******************************************************************************/
+
+QTMMenuWidget::QTMMenuWidget (QWidget* parent) : QWidget (parent) {
+  QTMAuxMenu m;
+  m.myInitStyleOption (&option);
+}
+
+void
+QTMMenuWidget::paintEvent(QPaintEvent* e) {
+  QPainter p (this);
+  option.rect = rect ();
+  style()->drawControl (QStyle::CE_MenuEmptyArea, &option, &p, this);
+  QWidget::paintEvent (e);
+}
 
 /******************************************************************************
  * QTMLazyMenu
  ******************************************************************************/
 
+QTMLazyMenu::QTMLazyMenu (promise<widget> _pm, QWidget* p, bool right)
+: QMenu (p), promise_widget (_pm), show_right (right) {
+  QObject::connect (this, SIGNAL (aboutToShow ()), this, SLOT (force ()));
+}
+
 void
-rerootActions (QWidget* dest, QWidget* src) {
-  if (src == NULL || dest == NULL) return;
-  QList<QAction *> list = dest->actions();
+QTMLazyMenu::showEvent (QShowEvent* e)
+{
+  if (show_right && parentWidget()) {
+    QPoint p = pos();
+    p.rx() += parentWidget()->width();
+    p.ry() -= parentWidget()->height();
+    move (p);
+  }
+  QMenu::showEvent (e);
+}
+
+/*! Sets the QTMLazyMenu as the menu for the QAction and makes its destruction
+ depend on that of the latter. */
+void
+QTMLazyMenu::attachTo (QAction* a) {
+  QObject::connect (a,  SIGNAL (destroyed (QObject*)),
+                    this, SLOT (destroy (QObject*)));
+  a->setMenu (this);
+}
+
+void
+QTMLazyMenu::transferActions (QWidget* from) {
+  if (from == NULL) return;
+  QList<QAction*> list = actions();
   while (!list.isEmpty()) {
-    QAction* a= list.takeFirst();
-    dest->removeAction (a);
-    //    delete a;
+    QAction* a = list.takeFirst();
+    removeAction (a);
     a->deleteLater();
   }
-  list = src->actions();
+  list = from->actions();
   while (!list.isEmpty()) {
-    QAction* a= list.takeFirst();
-    dest->addAction (a);
-    a->setParent (dest);
+    QAction* a = list.takeFirst();
+    addAction (a);
+    a->setParent (this);
   }
 }
 
 void
 QTMLazyMenu::force () {
-  if (DEBUG_QT_WIDGETS) cout << "Force lazy menu" << LF;
-  widget w = pm ();
-  QMenu* menu2 = concrete(w)->get_qmenu();
-  rerootActions (this, menu2);
-    //menu2->deleteLater();  // OK? Conflicting policies in qt_menu and qt_ui_element
+  QAction* a = concrete (promise_widget())->as_qaction();
+  transferActions (a->menu());
+  delete a;
+}
+
+void
+QTMLazyMenu::destroy (QObject* obj) {
+  (void) obj;
+  deleteLater();
 }
 
 
@@ -262,15 +373,13 @@ QTMLazyMenu::force () {
  * QTMInputTextWidgetHelper
  ******************************************************************************/
 
-QTMInputTextWidgetHelper::QTMInputTextWidgetHelper (qt_input_text_widget_rep* _wid) 
-: QObject(NULL), p_wid(abstract(_wid)), done(false) { }
-
-/*! Destructor.
- Removes reference to the helper in the texmacs widget. If needed the texmacs
- widget is automatically deleted.
- */
-QTMInputTextWidgetHelper::~QTMInputTextWidgetHelper() {
-  wid()->helper = NULL;
+QTMInputTextWidgetHelper::QTMInputTextWidgetHelper (qt_widget _wid,
+                                                    QTMLineEdit* le)
+: QObject (le), p_wid (_wid), done (false) {
+  ASSERT (le != NULL, "QTMInputTextWidgetHelper: expecting valid QTMLineEdit");
+  QObject::connect (le, SIGNAL (returnPressed ()), this, SLOT (commit ()));
+  QObject::connect (le, SIGNAL (focusOut (Qt::FocusReason)),
+                    this, SLOT (leave (Qt::FocusReason)));
 }
 
 void
@@ -294,19 +403,22 @@ QTMInputTextWidgetHelper::commit () {
 
     // HACK: restore focus to the main editor widget
   widget_rep* win = qt_window_widget_rep::widget_from_qwidget (le);
-  if (win && concrete(win)->type == qt_widget_rep::texmacs_widget)
-    concrete(get_canvas(win))->qwid->setFocus();
+  if (win && concrete (win)->type == qt_widget_rep::texmacs_widget)
+    concrete (get_canvas (win))->qwid->setFocus();
 
   if (win) apply();    // This is 0 inside a dialog => no command
 }
 
 /*! Executed after commit of the input field (enter) and when losing focus */
 void
-QTMInputTextWidgetHelper::leave () {
+QTMInputTextWidgetHelper::leave (Qt::FocusReason reason) {
   QTMLineEdit* le = qobject_cast <QTMLineEdit*> (sender());
   if (!le) return;
 
-  if (get_preference ("gui:line-input:autocommit") == "#t") {
+    // Don't autocommit if the user pressed the escape key
+    // (see QTMLineEdit::focusOut()).
+  if (reason != Qt::OtherFocusReason &&
+      get_preference ("gui:line-input:autocommit") == "#t") {
     done         = false;
     wid()->ok    = true;
     wid()->input = from_qstring (le->text());
@@ -314,73 +426,35 @@ QTMInputTextWidgetHelper::leave () {
     le->setText (to_qstring (wid()->input));
   }
 
-  widget_rep* win = qt_window_widget_rep::widget_from_qwidget(le);
+  widget_rep* win = qt_window_widget_rep::widget_from_qwidget (le);
   if (win) apply();    // This is 0 inside a dialog => no command
 }
 
-void
-QTMInputTextWidgetHelper::remove (QObject* obj) {
-  views.removeAll (qobject_cast<QTMLineEdit*> (obj));
-  if (views.count () == 0)
-    deleteLater();
-}
-
-void
-QTMInputTextWidgetHelper::add (QObject* obj) {
-  QTMLineEdit* le = qobject_cast<QTMLineEdit*> (obj);
-  if (le && !views.contains (le)) {
-    QObject::connect (le, SIGNAL (destroyed (QObject*)), this, SLOT (remove (QObject*)));
-    QObject::connect (le, SIGNAL (returnPressed ()),     this, SLOT (commit ()));
-    QObject::connect (le, SIGNAL (focusOut ()),          this, SLOT (leave ()));
-    views << le;
-  }
-}
-
-
 /******************************************************************************
  * QTMFieldWidgetHelper
  ******************************************************************************/
 
-QTMFieldWidgetHelper::QTMFieldWidgetHelper (qt_field_widget _wid) 
-  : QObject(NULL), wid(_wid), done(false) { }
-
-/*! Destructor.
- Removes reference to the helper in the texmacs widget. Deletion of the texmacs
- widget is automagic.
- */
-QTMFieldWidgetHelper::~QTMFieldWidgetHelper() {
-  wid->helper = NULL;
+QTMFieldWidgetHelper::QTMFieldWidgetHelper (qt_widget _wid, QComboBox* cb)
+: QObject (cb), wid (_wid), done (false) {
+  ASSERT (cb != NULL, "QTMFieldWidgetHelper: expecting valid QComboBox");
+  QObject::connect (cb, SIGNAL (editTextChanged (const QString&)),
+                    this, SLOT (commit (const QString&)));
 }
 
 void
 QTMFieldWidgetHelper::commit (const QString& qst) {
-  wid->input = scm_quote (from_qstring (qst));
+  static_cast<qt_field_widget_rep*> (wid.rep)->input =
+      scm_quote (from_qstring (qst));
 }
 
-void
-QTMFieldWidgetHelper::remove (QObject* obj) {
-  views.removeAll (qobject_cast<QTMComboBox*> (obj));
-  if (views.count () == 0)
-    deleteLater();
-}
-
-void
-QTMFieldWidgetHelper::add (QObject* obj) {
-  QComboBox* cb = qobject_cast<QComboBox*> (obj);
-  if (obj && !views.contains (cb)) {
-    QObject::connect (cb, SIGNAL (destroyed (QObject*)), this, SLOT (remove (QObject*)));
-    QObject::connect (cb, SIGNAL (editTextChanged (const QString&)), this, SLOT (commit (const QString&)));
-    views << cb;
-  }
-}
-
-
 /******************************************************************************
  * QTMLineEdit
  ******************************************************************************/
 
-QTMLineEdit::QTMLineEdit (QWidget* parent, string _ww, int style)
-: QLineEdit (parent), completing (false), ww (_ww) {
+QTMLineEdit::QTMLineEdit (QWidget* parent, string _type, string _ww,
+                          int style, command _cmd)
+  : QLineEdit (parent), completing (false),
+    type (_type), ww (_ww), cmd (_cmd), last_key (0) {
   if (style & WIDGET_STYLE_MINI) {
     setStyle (qtmstyle());
       // FIXME: we should remove this and let the scheme code decide.
@@ -391,11 +465,16 @@ QTMLineEdit::QTMLineEdit (QWidget* parent, string _ww, int style)
   }
   
     // just to be sure we don't capture the wrong keys in keyPressEvent
-  setCompleter(0);
+  setCompleter (0);
 
   qt_apply_tm_style (this, style);
 }
 
+bool
+QTMLineEdit::continuous () {
+  return type == "search" || starts (type, "replace-");
+}
+
 /*
  We need to reimplement the main event handler because we need the tab key for
  completions. (See Qt docs for QWidget::event())
@@ -403,92 +482,174 @@ QTMLineEdit::QTMLineEdit (QWidget* parent, string _ww, int style)
 bool
 QTMLineEdit::event (QEvent* ev) {
   if (ev->type() == QEvent::KeyPress)  // Handle ALL keys
-    keyPressEvent(static_cast<QKeyEvent*> (ev));
+    keyPressEvent (static_cast<QKeyEvent*> (ev));
   else
     return QWidget::event (ev);
-
   return true;
 }
 
+extern hashmap<int,string> qtkeymap;
+
 /*
  FIXME: This is a hideous mess...
  */
 void 
-QTMLineEdit::keyPressEvent(QKeyEvent* ev)
+QTMLineEdit::keyPressEvent (QKeyEvent* ev)
 {
   QCompleter* c = completer();
-
-  int key = (ev->key() == Qt::Key_Tab && ev->modifiers() & Qt::ShiftModifier)
+  
+  last_key = (ev->key() == Qt::Key_Tab && ev->modifiers() & Qt::ShiftModifier)
             ? Qt::Key_Backtab
             : ev->key();
-  
-  if (c) {
+
+  if (continuous ()) {
+    if ((last_key != Qt::Key_Tab || type == "replace-what") &&
+        (last_key != Qt::Key_Backtab || type == "replace-by") &&
+        last_key != Qt::Key_Down &&
+        last_key != Qt::Key_Up &&        
+        last_key != Qt::Key_Enter &&
+        last_key != Qt::Key_Return &&
+        last_key != Qt::Key_Escape &&
+        (ev->modifiers() & Qt::ControlModifier) == 0 &&
+        (ev->modifiers() & Qt::MetaModifier) == 0)
+      QLineEdit::keyPressEvent (ev);
+    string key= "none";
+    string s  = from_qstring (text());
+    if (last_key >= 32 && last_key <= 126) {
+      key= string ((char) last_key);
+      if (key[0] >= 'A' && key[0] <= 'Z')
+        if ((ev->modifiers() & Qt::ShiftModifier) == 0)
+          key[0]= (int) (key[0] + ((int) 'a') - ((int) 'A'));
+    }
+    if (qtkeymap->contains (last_key)) key= qtkeymap[last_key];
+    if ((ev->modifiers() & Qt::ShiftModifier) && N(key) > 1) key= "S-" * key;
+#ifdef Q_WS_MAC
+    if (ev->modifiers() & Qt::ControlModifier) key= "C-" * key;
+    if (ev->modifiers() & Qt::AltModifier) key= "none";
+    if (ev->modifiers() & Qt::MetaModifier) key= "M-" * key;
+#else
+    if (ev->modifiers() & Qt::ControlModifier) key= "M-" * key;
+    if (ev->modifiers() & Qt::AltModifier) key= "A-" * key;
+    if (ev->modifiers() & Qt::MetaModifier) key= "C-" * key;
+#endif
+    cmd (list_object (list_object (object (s), object (key))));
+    return;
+  }
+  else if (c) {
     int row = 0;
-    switch (key) {
+    switch (last_key) {
       case Qt::Key_Down:
         completing = true;
+        setCursorPosition (0);
         c->complete();
       case Qt::Key_Tab:
       {
+//        cout << "Completing= " << completing << LF;
+//        cout << "hasSelectedText= " << hasSelectedText() << LF;
+//        cout << "CursorPosition= " << cursorPosition() << LF;
+//        cout << "SelectionStart= " << selectionStart() << LF;
+        
         if (completing) {
+//          cout << "CompletionCount= " << c->completionCount() << LF;
           if (c->completionCount() > 1) {
-            if(! c->setCurrentRow (c->currentRow() + 1))
+            if (! c->setCurrentRow (c->currentRow() + 1))
               c->setCurrentRow (0);    // cycle
           } else {
-            setCursorPosition (text().length());
             completing = false;
+            setCursorPosition (text().length());
+            c->setCompletionPrefix ("");
               //c->popup()->hide();
           }
-        } else {
           if (hasSelectedText())
-            c->setCompletionPrefix (text().left (selectionStart()));
+            setCursorPosition (selectionStart());
+          if (c->currentCompletion() != "") {
+            int pos = cursorPosition();
+            setText (c->currentCompletion ());
+            setSelection (pos, text().length());
+          } else {
+            completing = false;
+            setSelection (0, text(). length());
+            c->setCompletionPrefix ("");
+          }
+        } else {
+          QString prefix;
+          if (hasSelectedText())
+            prefix = text().left (selectionStart());
           else
-            c->setCompletionPrefix (text().left (cursorPosition()));
-
-            // If there are no completions, send the key up for tab navigation
+            prefix = text().left (cursorPosition());
+          c->setCompletionPrefix (prefix);
+//          cout << "prefix= " << from_qstring (prefix) << LF;
+//          cout << "CompletionCount= " << c->completionCount() << LF;
+          
+            // If there are no completions, go to the end of the line or
+            // send the key up for tab navigation
           if (c->completionCount() == 0 ||
               (c->completionCount() == 1 && c->currentCompletion() == text())) {
-            QLineEdit::keyPressEvent (ev);
+            if (c->popup() && c->popup()->isVisible()) {
+              setCursorPosition (text().length());
+              c->popup()->hide();
+            } else if (cursorPosition() == text().length()) {
+              QLineEdit::keyPressEvent (ev);
+            } else {
+              setCursorPosition (text().length());
+            }
             return;
           }
-          
+
           completing = true;
+            // hack: advance one completion (needed after tab navigation)
+          if (c->currentCompletion() == text()) {
+            clear();
+            if (! c->setCurrentRow (c->currentRow() + 1))
+              c->setCurrentRow (0);    // cycle
+          }
           c->complete();
         }
-        if (hasSelectedText())
-          setCursorPosition (selectionStart());
-        if (c->currentCompletion() != "") {
-          int pos = cursorPosition();
-          setText (c->currentCompletion ());
-          setSelection (pos, text().length() - 1);
-        } else {
-          completing = false;
-          setSelection (0, text(). length() - 1);
-        }
-
         ev->accept();
       }
         return;
           // This is different on purpose: when "back-completing" suggested text
           // we want to display the previous entry to the one suggested
       case Qt::Key_Up:
+        completing = true;
+        setCursorPosition (0);
+        c->complete();
       case Qt::Key_Backtab:
       {
-        completing = true;
-        c->complete();
-        row = c->currentRow();
-        if(! c->setCurrentRow (row - 1))
-          c->setCurrentRow (c->completionCount() - 1);    // cycle
-        if (c->currentCompletion() != "") {
-          int pos;
+        if (!completing) {
           if (hasSelectedText())
-            pos = selectionStart();
+            c->setCompletionPrefix (text().left (selectionStart()));
           else
-            pos = cursorPosition();
-          setText (c->currentCompletion ());
-          setSelection (pos, text().length() - 1);
+            c->setCompletionPrefix (text().left (cursorPosition()));
+            // If there are no completions, go to the end of the line or
+            // send the key up for tab navigation
+          if (c->completionCount() == 0 ||
+              (c->completionCount() == 1 && c->currentCompletion() == text())) {
+            if (c->popup() && c->popup()->isVisible()) {
+              setCursorPosition (text().length());
+              c->popup()->hide();
+            } else if (cursorPosition() == text().length()) {
+              QLineEdit::keyPressEvent (ev);
+            } else {
+              setCursorPosition (text().length());
+            }
+            return;
+          }
+          completing = true;
+          c->complete();
         } else {
-            // TODO: blink somehow
+          row = c->currentRow();
+          if (! c->setCurrentRow (row - 1))
+            c->setCurrentRow (c->completionCount() - 1);    // cycle
+          if (c->currentCompletion() != "") {
+            int pos;
+            if (hasSelectedText())
+              pos = selectionStart();
+            else
+              pos = cursorPosition();
+            setText (c->currentCompletion ());
+            setSelection (pos, text().length());
+          }
         }
         ev->accept();
       }
@@ -499,27 +660,33 @@ QTMLineEdit::keyPressEvent(QKeyEvent* ev)
           setCursorPosition (text().length());
           c->popup()->hide();
         } else if (completing) {
+          completing = false;
           setText (c->currentCompletion());
           setCursorPosition (text().length());
+          c->setCompletionPrefix ("");
         } else {
-          emit returnPressed();
+          completing = false;
+          c->setCompletionPrefix ("");
+          QLineEdit::keyPressEvent (ev);
+          return;
         }
-        completing = false;
         ev->accept();
         return;
       case Qt::Key_Escape:
-        if (completing) {
+        if (completing && c->completionMode() == QCompleter::PopupCompletion) {
           if (c->popup()) c->popup()->hide();
           completing = false;
         } else {
           emit editingFinished();
           ev->accept();
           if (parentWidget())        // HACK to return focus to the main editor widget
-            parentWidget()->setFocus();
+            parentWidget()->setFocus ();
         }
+        c->setCompletionPrefix ("");
         return;
       default:
         completing = false;
+        c->setCompletionPrefix ("");
         QLineEdit::keyPressEvent (ev);
         return;
     }
@@ -530,7 +697,7 @@ QTMLineEdit::keyPressEvent(QKeyEvent* ev)
 
 QSize
 QTMLineEdit::sizeHint () const {
-  return qt_decode_length(ww, "", QLineEdit::sizeHint(), fontMetrics());
+  return qt_decode_length (ww, "", QLineEdit::sizeHint(), fontMetrics());
 }
 
 void 
@@ -544,18 +711,21 @@ QTMLineEdit::focusInEvent (QFocusEvent* ev)
 void
 QTMLineEdit::focusOutEvent (QFocusEvent* ev)
 {
-  emit focusOut();
+  if (!continuous ()) {
+    Qt::FocusReason reason =
+      (last_key != Qt::Key_Escape) ? ev->reason() : Qt::OtherFocusReason;
+    emit focusOut (reason);
+  }
   QLineEdit::focusOutEvent (ev);
 }
 
 
-
 /******************************************************************************
  * QTMTabWidget
  ******************************************************************************/
 
-QTMTabWidget::QTMTabWidget(QWidget *p) : QTabWidget(p) {
-  QObject::connect(this, SIGNAL(currentChanged(int)), this, SLOT(resizeOthers(int)));
+QTMTabWidget::QTMTabWidget (QWidget *p) : QTabWidget(p) {
+  QObject::connect (this, SIGNAL (currentChanged (int)), this, SLOT (resizeOthers (int)));
 }
 
 /*! Resizes the widget to the size of the tab given by the index.
@@ -566,12 +736,12 @@ QTMTabWidget::QTMTabWidget(QWidget *p) : QTabWidget(p) {
  case we must update the fixed size to reflect the change of tab.
  */
 void
-QTMTabWidget::resizeOthers(int current) {
-  for(int i = 0; i < count(); ++i) {
+QTMTabWidget::resizeOthers (int current) {
+  for (int i = 0; i < count(); ++i) {
     if (i != current)
-      widget(i)->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
+      widget(i)->setSizePolicy (QSizePolicy::Ignored, QSizePolicy::Ignored);
     else
-      widget(i)->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
+      widget(i)->setSizePolicy (QSizePolicy::Minimum, QSizePolicy::Minimum);
   }
   
     // FIXME? this could loop indefinitely if parents are cyclic.
@@ -580,10 +750,11 @@ QTMTabWidget::resizeOthers(int current) {
     p->adjustSize();
     p = p->parentWidget();
   }
+  p->adjustSize();
 
-  if (window()->minimumSize()!=QSize(0,0) && 
-      window()->maximumSize() != QSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX))
-    window()->setFixedSize(window()->sizeHint());
+  if (window()->minimumSize()!=QSize (0,0) && 
+      window()->maximumSize() != QSize (QWIDGETSIZE_MAX, QWIDGETSIZE_MAX))
+    window()->setFixedSize (window()->sizeHint());
 }
 
 
@@ -594,25 +765,24 @@ QTMTabWidget::resizeOthers(int current) {
 
 widget make_menu_widget (object wid);
 
-QTMRefreshWidget::QTMRefreshWidget (string _tmwid, string _kind):
-  QWidget (), tmwid (_tmwid), kind (_kind),
-  curobj (false), cur (), qwid(NULL),
-  cache (widget ())
+QTMRefreshWidget::QTMRefreshWidget (qt_widget _tmwid, string _strwid, string _kind)
+: QWidget (), strwid (_strwid), kind (_kind),
+  curobj (false), cur (), tmwid (_tmwid), qwid (NULL), cache (widget ())
 {   
-  QObject::connect(the_gui->gui_helper, SIGNAL(tmSlotRefresh(string)),
-                   this, SLOT(doRefresh(string)));
+  QObject::connect (the_gui->gui_helper, SIGNAL (tmSlotRefresh (string)),
+                   this, SLOT (doRefresh (string)));
   QVBoxLayout* l = new QVBoxLayout (this);
   l->setContentsMargins (0, 0, 0, 0);
   l->setMargin (0);
   setLayout (l);
   
-  doRefresh("init");
+  doRefresh ("init");
 }
 
 bool
 QTMRefreshWidget::recompute (string what) {
   if (what != "init" && kind != "any" && kind != what) return false;
-  string s = "'(vertical (link " * tmwid * "))";
+  string s = "'(vertical (link " * strwid * "))";
   eval ("(lazy-initialize-force)");
   object xwid = call ("menu-expand", eval (s));
   
@@ -625,6 +795,7 @@ QTMRefreshWidget::recompute (string what) {
     curobj = xwid;
     object uwid = eval (s);
     cur = make_menu_widget (uwid);
+    tmwid->add_child (cur); // FIXME?! Is this ok? what when we refresh?
     cache (xwid) = cur;
     return true;
   }
@@ -676,6 +847,83 @@ QTMRefreshWidget::doRefresh (string kind) {
 }
 
 
+/******************************************************************************
+ * QTMRefreshableWidget
+ ******************************************************************************/
+
+QTMRefreshableWidget::QTMRefreshableWidget (qt_widget _tmwid, object _prom, string _kind)
+: QWidget (), prom (_prom), kind (_kind),
+  curobj (false), cur (), tmwid (_tmwid), qwid (NULL)
+{   
+  QObject::connect (the_gui->gui_helper, SIGNAL (tmSlotRefresh (string)),
+                   this, SLOT (doRefresh (string)));
+  QVBoxLayout* l = new QVBoxLayout (this);
+  l->setContentsMargins (0, 0, 0, 0);
+  l->setMargin (0);
+  setLayout (l);
+  
+  doRefresh ("init");
+}
+
+bool
+QTMRefreshableWidget::recompute (string what) {
+  if (what != "init" && kind != "any" && kind != what) return false;
+  eval ("(lazy-initialize-force)");
+  object xwid = call (prom);
+  if (curobj == xwid) return false;
+  if (!is_widget (xwid)) return false;
+  curobj= xwid;
+  cur= as_widget (xwid);
+  tmwid->add_child (cur); // FIXME?! Is this ok? what when we refresh?
+  return true;
+}
+
+/*
+void
+QTMRefreshableWidget::deleteLayout (QLayout* l) {
+  if (!l)
+    return;
+
+  QLayoutItem* item;
+  while ((item = l->takeAt(0)) != 0) {
+    if (item->widget()) {
+        //qDebug() << "Deleting widget: " << item->widget();
+      l->removeWidget (item->widget());
+      item->widget()->setParent (NULL);
+      delete item->widget();
+    }	else if (item->layout()) {
+        //qDebug() << "Deleting layout: " << item->layout();
+      item->layout()->setParent (NULL);
+      deleteLayout (item->layout());
+    }
+  }
+
+  delete l;
+}
+*/
+
+void
+QTMRefreshableWidget::doRefresh (string kind) {
+  if (recompute (kind)) {
+    if (qwid) qwid->setParent (NULL);
+    delete qwid;
+    qwid = concrete (cur)->as_qwidget();
+    qwid->setParent (this);
+
+    delete layout()->takeAt(0);
+    layout()->addWidget (qwid);
+    update();
+    
+      // Tell the window to fix its size to the new one if we had it fixed to
+      // begin with (this is indicated by minimum and maximum sizes set to 
+      // values other than the default)
+    if (window()->minimumSize() != QSize (0,0) &&
+        window()->maximumSize() != QSize (QWIDGETSIZE_MAX, QWIDGETSIZE_MAX))
+      window()->setFixedSize (window()->sizeHint());  
+  }
+}
+
+
 /******************************************************************************
  * QTMComboBox
  ******************************************************************************/
@@ -683,17 +931,17 @@ QTMRefreshWidget::doRefresh (string kind) {
 QTMComboBox::QTMComboBox (QWidget* parent) : QComboBox (parent) {
     ///// Obtain the minimum vertical size
   QComboBox cb;
-  cb.setSizeAdjustPolicy(AdjustToContents);
-  cb.addItem("");
+  cb.setSizeAdjustPolicy (AdjustToContents);
+  cb.addItem ("");
   minSize = cb.sizeHint();  // we'll just keep the height
   
     ///// Add width of the arrow button
   QStyleOptionComboBox opt;
-  opt.initFrom(&cb);
+  opt.initFrom (&cb);
   opt.activeSubControls = QStyle::SC_ComboBoxArrow;
   QRect r = style()->subControlRect (QStyle::CC_ComboBox, &opt,
                                      QStyle::SC_ComboBoxArrow, &cb);
-  minSize.setWidth(r.width());
+  minSize.setWidth (r.width());
 }
 
 /*! Add items and fix the ComboBox size using texmacs length units.
@@ -706,14 +954,14 @@ QTMComboBox::QTMComboBox (QWidget* parent) : QComboBox (parent) {
  */
 void
 QTMComboBox::addItemsAndResize (const QStringList& texts, string ww, string hh) {
-  QComboBox::addItems(texts);
+  QComboBox::addItems (texts);
   
     ///// Calculate the minimal contents size:
   calcSize = QApplication::globalStrut ();
   const QFontMetrics& fm = fontMetrics ();
   
   for (int i = 0; i < count(); ++i) {
-    QRect br = fm.boundingRect(itemText(i));
+    QRect br = fm.boundingRect (itemText(i));
     calcSize.setWidth (qMax (calcSize.width(), br.width()));
     calcSize.setHeight (qMax (calcSize.height(), br.height()));
   }
@@ -723,7 +971,7 @@ QTMComboBox::addItemsAndResize (const QStringList& texts, string ww, string hh)
   calcSize.setHeight (qMax (calcSize.height(), minSize.height()));
   calcSize.rwidth() += minSize.width();
   
-  setFixedSize(calcSize);
+  setFixedSize (calcSize);
 }
 
 /*
@@ -737,7 +985,7 @@ QTMComboBox::event (QEvent* ev) {
     if (k->key() == Qt::Key_Up || k->key() == Qt::Key_Down)
       showPopup();
     else if (k->key() != Qt::Key_Escape) // HACK: QTMLineEdit won't need this
-      lineEdit()->event(ev);             // but we do.
+      lineEdit()->event (ev);             // but we do.
     else
       return false;
   } else
@@ -832,7 +1080,7 @@ QTMListView::QTMListView (const command& cmd,
 
   for (int i = 0; i < model()->rowCount(); ++i) {
     QModelIndex item = model()->index (i, 0);
-    if (selections.contains (model()->data(item, Qt::DisplayRole).toString(), Qt::CaseSensitive))
+    if (selections.contains (model()->data (item, Qt::DisplayRole).toString(), Qt::CaseSensitive))
       selectionModel()->select (item, QItemSelectionModel::SelectCurrent);
   }
 
@@ -865,3 +1113,42 @@ QTMListView::selectionChanged (const QItemSelection& c, const QItemSelection& p)
   QListView::selectionChanged (c, p);
   emit selectionChanged (c);
 }
+
+
+/******************************************************************************
+ * QTMTreeView
+ ******************************************************************************/
+
+QTMTreeView::QTMTreeView (command cmd, tree data, const tree& roles, QWidget* p)
+: QTreeView (p), _t (data), _cmd (cmd) {
+  setModel (QTMTreeModel::instance (_t, roles));
+  setUniformRowHeights (true);  // assuming we display only text.
+  setHeaderHidden (true);       // for now...
+  QObject::connect (this, SIGNAL (pressed (const QModelIndex&)),
+                    this,   SLOT (callOnClick (const QModelIndex&)));
+}
+
+void
+QTMTreeView::callOnClick (const QModelIndex& index) {
+  object arguments = list_object ((int)QApplication::mouseButtons());
+    // docs state the index is valid, no need to check
+  QVariant d = tmModel()->data (index, QTMTreeModel::CommandRole);
+    // If there's no CommandRole, we return the subtree by default
+  if (!d.isValid() || !d.canConvert (QVariant::String))
+    arguments = cons (tmModel()->item_from_index (index), arguments);
+  else
+    arguments = cons (from_qstring (d.toString()), arguments);
+  int cnt = QTMTreeModel::TMUserRole;
+  d = tmModel()->data (index, cnt);
+  while (d.isValid() && d.canConvert (QVariant::String)) {
+    arguments = cons (from_qstring (d.toString()), arguments);
+    d = tmModel()->data (index, ++cnt);
+  }
+  _cmd (arguments);
+}
+
+inline QTMTreeModel*
+QTMTreeView::tmModel() const {
+  return static_cast<QTMTreeModel*> (model());
+}
+
