@@ -27,35 +27,31 @@
 #include <QByteArray>
 
 /*!
- * @param _cmd  Scheme closure to execute after the dialog is closed.
- * @param _type What kind of dialog to show. Can be one of "image", "directory",
- *              or any of the supported file formats: "texmacs", "tmml",
-                "postscript", etc. See perform_dialog()
+  \param _cmd  Scheme closure to execute after the dialog is closed.
+  \param _type What kind of dialog to show. Can be one of "image", "directory",
+               or any of the supported file formats: "texmacs", "tmml",
+               "postscript", etc. See perform_dialog()
  */
 qt_chooser_widget_rep::qt_chooser_widget_rep (command _cmd, string _type, bool _save)
  : qt_widget_rep (file_chooser), cmd (_cmd), save (_save),
    position (coord2 (0, 0)), size (coord2 (100, 100)), file ("")
 {
   if (DEBUG_QT_WIDGETS)
-    cout << "qt_chooser_widget_rep::qt_chooser_widget_rep type=\""
-         << type << "\" save=\"" << save << "\"" << LF;
+    debug_widgets << "qt_chooser_widget_rep::qt_chooser_widget_rep type=\""
+                  << type << "\" save=\"" << save << "\"" << LF;
   if (! set_type (_type))
     set_type ("generic");
 }
 
-qt_chooser_widget_rep::~qt_chooser_widget_rep() {}
-
-
 void
 qt_chooser_widget_rep::send (slot s, blackbox val) {
-
   switch (s) {
     case SLOT_VISIBILITY:
     {   
       check_type<bool> (val, s);
       bool flag = open_box<bool> (val);
       (void) flag;
-      NOT_IMPLEMENTED
+      NOT_IMPLEMENTED("qt_chooser_widget::SLOT_VISIBILITY");
     }
       break;
     case SLOT_SIZE:
@@ -73,23 +69,18 @@ qt_chooser_widget_rep::send (slot s, blackbox val) {
     case SLOT_STRING_INPUT:
       check_type<string>(val, s);
       if (DEBUG_QT_WIDGETS)
-        cout << "\tString input: " << open_box<string> (val) << LF;
-      NOT_IMPLEMENTED
+        debug_widgets << "\tString input: " << open_box<string> (val) << LF;
+      NOT_IMPLEMENTED("qt_chooser_widget::SLOT_STRING_INPUT");
       break;
     case SLOT_INPUT_TYPE:
       check_type<string>(val, s);
       set_type (open_box<string> (val));
       break;
-#if 0
-    case SLOT_INPUT_PROPOSAL:
-        //send_string (THIS, "default", val);
-      NOT_IMPLEMENTED
-      break;
-#endif
     case SLOT_FILE:
         //send_string (THIS, "file", val);
       check_type<string>(val, s);
-      if (DEBUG_QT_WIDGETS) cout << "\tFile: " << open_box<string> (val) << LF;
+      if (DEBUG_QT_WIDGETS)
+        debug_widgets << "\tFile: " << open_box<string> (val) << LF;
       file = open_box<string> (val);
       break;
     case SLOT_DIRECTORY:
@@ -102,14 +93,14 @@ qt_chooser_widget_rep::send (slot s, blackbox val) {
       qt_widget_rep::send (s, val);
   }
   if (DEBUG_QT_WIDGETS)
-    cout << "qt_chooser_widget_rep: sent " << slot_name (s) 
-         << "\t\tto widget\t"      << type_as_string() << LF;
+    debug_widgets << "qt_chooser_widget_rep: sent " << slot_name (s) 
+                  << "\t\tto widget\t"      << type_as_string() << LF;
 }
 
 blackbox
 qt_chooser_widget_rep::query (slot s, int type_id) {
   if (DEBUG_QT_WIDGETS)
-    cout << "qt_chooser_widget_rep::query " << slot_name(s) << LF;
+    debug_widgets << "qt_chooser_widget_rep::query " << slot_name(s) << LF;
   switch (s) {
     case SLOT_POSITION:
     {
@@ -124,7 +115,7 @@ qt_chooser_widget_rep::query (slot s, int type_id) {
     case SLOT_STRING_INPUT:
     {
       check_type_id<string> (type_id, s);
-      if (DEBUG_QT_WIDGETS) cout << "\tString: " << file << LF;
+      if (DEBUG_QT_WIDGETS) debug_widgets << "\tString: " << file << LF;
       return close_box<string> (file);
     }
     default:
@@ -135,7 +126,7 @@ qt_chooser_widget_rep::query (slot s, int type_id) {
 widget
 qt_chooser_widget_rep::read (slot s, blackbox index) {
   if (DEBUG_QT_WIDGETS)
-    cout << "qt_chooser_widget_rep::read " << slot_name(s) << LF;
+    debug_widgets << "qt_chooser_widget_rep::read " << slot_name(s) << LF;
   switch (s) {
     case SLOT_WINDOW:
       check_type_void (index, s);
@@ -154,6 +145,10 @@ qt_chooser_widget_rep::read (slot s, blackbox index) {
   }
 }
 
+/*!
+ @note: name is a unique identifier for the window, but for this widget we
+ identify it with the window title. This is not always the case.
+ */
 widget
 qt_chooser_widget_rep::plain_window_widget (string s, command q)
 {
@@ -162,7 +157,6 @@ qt_chooser_widget_rep::plain_window_widget (string s, command q)
   return this;
 }
 
-
 bool
 qt_chooser_widget_rep::set_type (const string& _type)
 {
@@ -183,7 +177,8 @@ qt_chooser_widget_rep::set_type (const string& _type)
     nameFilter = to_qstring (translate ("Image file"));
   } else {
     if (DEBUG_STD)
-      cout << "qt_chooser_widget: IGNORING unknown format " << _type << LF;
+      debug_widgets << "qt_chooser_widget: IGNORING unknown format "
+                    << _type << LF;
     return false;
   }
 
@@ -243,7 +238,7 @@ qt_chooser_widget_rep::perform_dialog () {
     QStringList filters;
     if (nameFilter != "")
       filters << nameFilter;
-    filters << to_qstring (translate ("All files (*.*)"));
+    filters << to_qstring (translate ("All files (*)"));
     dialog->setNameFilters (filters);
   }
 #endif
@@ -252,6 +247,7 @@ qt_chooser_widget_rep::perform_dialog () {
   QSize   sz = dialog->sizeHint();
   QPoint pos = to_qpoint (position);
   QRect r;
+
   r.setSize (sz);
   r.moveCenter (pos);
   dialog->setGeometry (r);
@@ -273,7 +269,7 @@ qt_chooser_widget_rep::perform_dialog () {
 #if !defined(Q_WS_MAC) && !defined(Q_WS_WIN)
         file = "(list " * file * imgdialog->getParamsAsString () * ")";
 #else
-        QPixmap pic (fileNames[0]);
+        QPixmap pic (fileNames.first());
         string params;
           // HACK: which value should we choose here?
         int ww = (get_current_editor()->get_page_width () / PIXEL) / 3;
