@@ -20,9 +20,11 @@
   (texmacs-version-release texmacs_version (string string))
   (version-before? version_inf (bool string string))
   (updater-supported? updater_supported (bool))
+  (updater-running? updater_is_running (bool))
   (updater-check-background updater_check_background (bool))
   (updater-check-foreground updater_check_foreground (bool))
   (updater-last-check updater_last_check (long))
+  (updater-set-appcast updater_set_appcast (bool url))
   (updater-set-interval updater_set_interval (bool int))
   (updater-set-automatic updater_set_automatic (bool bool))
   (os-win32? os_win32 (bool))
@@ -38,6 +40,8 @@
   (tm-errput tm_errput (void string))
   (win32-display win32_display (void string))
   (cpp-error cpp_error (void))
+  (supports-native-pdf? supports_native_pdf (bool))
+  (supports-ghostscript? supports_ghostscript (bool))
   (rescue-mode? in_rescue_mode (bool))
   (scheme-dialect scheme_dialect (string))
   (get-texmacs-path get_texmacs_path (url))
@@ -58,12 +62,15 @@
   (system-wait system_wait (void string string))
   (set-latex-command set_latex_command (void string))
   (set-bibtex-command set_bibtex_command (void string))
+  (number-latex-errors number_latex_errors (int url))
+  (number-latex-pages number_latex_pages (int url))
   (math-symbol-group math_symbol_group (string string))
   (math-group-members math_group_members (array_string string))
   (math-symbol-type math_symbol_type (string string))
   (object->command as_command (command object))
   (exec-delayed exec_delayed (void object))
   (exec-delayed-pause exec_delayed_pause (void object))
+  (protected-call protected_call (void object))
   (notify-preferences-booted notify_preferences_booted (void))
   (cpp-has-preference? has_user_preference (bool string))
   (cpp-get-preference get_user_preference (string string string))
@@ -86,6 +93,8 @@
   (get-author get_author (double))
   (debug-set debug_set (void string bool))
   (debug-get debug_get (bool string))
+  (get-debug-messages get_debug_messages (tree string int))
+  (clear-debug-messages clear_debug_messages (void))
   (cout-buffer cout_buffer (void))
   (cout-unbuffer cout_unbuffer (string))
   (mark-new new_marker (double))
@@ -116,6 +125,8 @@
   (font-database-search font_database_search (array_string string string))
   (font-database-characteristics
    font_database_characteristics (array_string string string))
+  (font-database-substitutions
+   font_database_substitutions (scheme_tree string))
   (font-family->master family_to_master (string string))
   (font-master->families master_to_families (array_string string))
   (font-master-features master_features (array_string string))
@@ -142,6 +153,7 @@
   (search-font-families search_font_families (array_string array_string))
   (search-font-styles search_font_styles (array_string string array_string))
   (logical-font-patch patch_font (array_string array_string array_string))
+  (logical-font-substitute apply_substitutions (array_string array_string))
   (font-family-main main_family (string string))
 
   ;; routines for images
@@ -170,6 +182,9 @@
   (tree-append tree_append (tree tree tree))
   (tree-right-index right_index (int tree))
   (tree-label-extension? is_extension (bool tree_label))
+  (tree-label-macro? is_macro (bool tree_label))
+  (tree-label-parameter? is_parameter (bool tree_label))
+  (tree-label-type get_tag_type (string tree_label))
   (tree-multi-paragraph? is_multi_paragraph (bool tree))
   (tree-simplify simplify_correct (tree tree))
   (tree-minimal-arity minimal_arity (int tree))
@@ -187,12 +202,14 @@
   (tree-child-long-name get_child_long_name (string content int))
   (tree-child-type get_child_type (string content int))
   (tree-child-env get_env_child (tree content int string content))
+  (tree-descendant-env get_env_descendant (tree content path content))
   (tree-load-inclusion load_inclusion (tree url))
   (tree-as-string tree_as_string (string content))
   (tree-extents tree_extents (tree content))
   (tree-empty? is_empty (bool content))
   (tree-is-buffer? admits_edit_observer (bool tree))
   (tree-search-sections search_sections (array_tree tree))
+  (tree-search-tree search (array_path content content path int))
   (tag-minimal-arity minimal_arity (int tree_label))
   (tag-maximal-arity maximal_arity (int tree_label))
   (tag-possible-arity? correct_arity (bool tree_label int))
@@ -231,6 +248,7 @@
   (math-status-reset math_status_reset (void))
 
   ;; paths
+  (path-strip strip (path path path))
   (path-inf? path_inf (bool path path))
   (path-inf-eq? path_inf_eq (bool path path))
   (path-less? path_less (bool path path))
@@ -295,6 +313,10 @@
   (string-quote scm_quote (string string))
   (string-unquote scm_unquote (string string))
   (string-convert convert (string string string string))
+  (encode-base64 encode_base64 (string string))
+  (decode-base64 decode_base64 (string string))
+  (sourcecode->cork sourcecode_to_cork (string string))
+  (cork->sourcecode cork_to_sourcecode (string string))
   (utf8->cork utf8_to_cork (string string))
   (cork->utf8 cork_to_utf8 (string string))
   (utf8->t2a utf8_to_t2a (string string))
@@ -310,6 +332,10 @@
   (integer->hexadecimal as_hexadecimal (string int))
   (integer->padded-hexadecimal as_hexadecimal (string int int))
   (hexadecimal->integer from_hexadecimal (int string))
+  
+  ; Program bracket matching
+  (find-left-bracket find_left_bracket (path path string string))
+  (find-right-bracket find_right_bracket (path path string string))
 
   ;; routines for strings in the TeXmacs encoding
   (string->tmstring tm_encode (string string))
@@ -321,6 +347,7 @@
   (list->tmstring tm_recompose (string array_string))
   (string-next tm_char_next (int string int))
   (string-previous tm_char_previous (int string int))
+  (tmstring-split tm_string_split (array_string string))
 
   ;; Packrat grammar and parsing tools
   (packrat-define packrat_define (void string string tree))
@@ -345,13 +372,20 @@
   (parse-latex parse_latex (tree string))
   (parse-latex-document parse_latex_document (tree string))
   (latex->texmacs latex_to_tree (tree tree))
-  (cpp-latex-document->texmacs latex_document_to_tree (tree string bool bool))
+  (cpp-latex-document->texmacs latex_document_to_tree (tree string bool))
   (latex-class-document->texmacs latex_class_document_to_tree (tree string))
+  (tracked-latex->texmacs tracked_latex_to_texmacs (tree string bool))
+  (conservative-texmacs->latex conservative_texmacs_to_latex
+                               (string content object))
+  (tracked-texmacs->latex tracked_texmacs_to_latex (string content object))
+  (conservative-latex->texmacs conservative_latex_to_texmacs (tree string bool))
   (parse-xml parse_xml (scheme_tree string))
   (parse-html parse_html (scheme_tree string))
   (parse-bib parse_bib (tree string))
   (upgrade-tmml tmml_upgrade (tree scheme_tree))
   (upgrade-mathml upgrade_mathml (tree content))
+  (vernac->texmacs vernac_to_tree (tree string))
+  (vernac-document->texmacs vernac_document_to_tree (tree string))
 
   ;; routines for urls
   (url->url url (url url))
@@ -385,6 +419,7 @@
   (url-tail tail (url url))
   (url-format file_format (string url))
   (url-suffix suffix (string url))
+  (url-basename basename (string url))
   (url-glue glue (url url string))
   (url-unglue unglue (url url int))
   (url-relative relative (url url url))
@@ -421,6 +456,8 @@
   (system-search-score search_score (int url array_string))
   (system-1 system (void string url))
   (system-2 system (void string url url))
+  (system-url->string sys_concretize (string url))
+  (url-grep grep (url string url))
 
   ;; Persistent data
   (persistent-set persistent_set (void url string string))
@@ -477,6 +514,7 @@
   (widget-choices choice_widget (widget command array_string array_string))
   (widget-filtered-choice choice_widget
     (widget command array_string string string))
+  (widget-tree-view tree_view_widget (widget command tree tree))
   (widget-xpm xpm_widget (widget url))
   (widget-box box_widget (widget scheme_tree string int bool bool))
   (widget-glue glue_widget (widget bool bool int int))
@@ -488,21 +526,24 @@
   (widget-icon-tabs icon_tabs_widget (widget array_url array_widget
                                              array_widget))
   (widget-scrollable user_canvas_widget (widget widget int))
-  (widget-resize resize_widget (widget widget int string string
+  (widget-resize resize_widget (widget widget int string string string string
                                        string string string string))
   (widget-hsplit hsplit_widget (widget widget widget))
   (widget-vsplit vsplit_widget (widget widget widget))
   (widget-texmacs-output texmacs_output_widget (widget content content))
-  (widget-texmacs-input texmacs_input_widget (widget content content 
-                                                     command bool))
+  (widget-texmacs-input texmacs_input_widget (widget content content url))
   (widget-ink ink_widget (widget command))
   (widget-refresh refresh_widget (widget string string))
+  (widget-refreshable refreshable_widget (widget object string))
   (object->promise-widget as_promise_widget (promise_widget object))
   (tree-bounding-rectangle get_bounding_rectangle (array_int tree))
+  (widget-size get_widget_size (array_int widget))
   (show-balloon show_help_balloon (void widget int int))
   (get-style-menu get_style_menu (object))
+  (hidden-package? hidden_package (bool string))
   (get-add-package-menu get_add_package_menu (object))
   (get-remove-package-menu get_remove_package_menu (object))
+  (get-toggle-package-menu get_toggle_package_menu (object))
   (refresh-now windows_refresh (void string))
 
   ;; buffers
@@ -535,6 +576,8 @@
   (tree-import-loaded import_loaded_tree (tree string url string))
   (tree-import import_tree (tree url string))
   (tree-export export_tree (bool tree url string))
+  (tree-load-style load_style_tree (tree string))
+  (buffer-focus focus_on_buffer (bool url))
 
   (view-list get_all_views (array_url))
   (buffer->views buffer_to_views (array_url url))
@@ -563,8 +606,8 @@
   (open-window-geometry open_window (url content))
   (clone-window clone_window (void))
   (buffer-close kill_buffer (void url))
-  (kill-window kill_window (void))
-  (kill-window-and-buffer kill_window_and_buffer (void))
+  (kill-window kill_window (void url))
+  (kill-current-window-and-buffer kill_current_window_and_buffer (void))
 
   (project-attach project_attach (void string))
   (project-detach project_attach (void))
