@@ -499,14 +499,17 @@ as_string (url u, int type) {
         else return s1 * sep * s2 (0, N(s2) - 1);
       }
       else {
-        if (type == URL_SYSTEM) return s2;
+        if (type == URL_SYSTEM) return "\\\\" * s2;
         else return s1 * sep * s2;
       }
     }
     if (is_root (u[1]) && type != URL_SYSTEM && N(s2) >= 2 && is_alpha (s2[0]) && s2[1] == ':')
       return s1 * sep * s2 (0, 1) * s2 (2, N(s2));
-    if (is_root (u[1]) && stype == URL_SYSTEM)
-      return s2;
+    if (is_root (u[1]) && stype == URL_SYSTEM) {
+      if (N(s2) >= 2 && is_alpha (s2[0]) && s2[1] == ':') return s2; 
+      else return "\\\\" * s2; 
+    }
+
 #endif
     return s1 * sep * s2;
   }
@@ -565,14 +568,6 @@ tail (url u) {
   return u;
 }
 
-string
-basename (url u, string suf) {
-  string s= as_string (tail (u));
-  if (suf != "" && N(s) > N(suf) && suf == s(N(s)-N(suf),N(s)))
-    return s(0, N(s)-N(suf));
-  return s;
-}
-
 string
 suffix (url u) {
   u= tail (u);
@@ -589,13 +584,28 @@ suffix (url u) {
   return "";
 }
 
+string
+basename (url u, string suf) {
+  string s= as_string (tail (u));
+  if (suf != "" && N(s) > N(suf) && suf == s(N(s)-N(suf),N(s)))
+    return s(0, N(s)-N(suf));
+  return s;
+}
+
+string
+basename (url u) {
+  string s= suffix (u);
+  if (N(s) != 0) s= "." * s;
+  return basename (u, s);
+}
+
 url
 glue (url u, string s) {
   if (is_atomic (u)) return as_url (tree (u->t->label * s));
   if (is_concat (u)) return u[1] * glue (u[2], s);
   if (is_or (u)) return glue (u[1], s) | glue (u[2], s);
-  cerr << "\nu= " << u << "\n";
-  cerr << "s= " << s << "\n";
+  failed_error << "u= " << u << "\n";
+  failed_error << "s= " << s << "\n";
   FAILED ("can't glue string to url");
   return u;
 }
@@ -606,8 +616,8 @@ unglue (url u, int nr) {
     return as_url (tree (u->t->label (0, max (N(u->t->label) - nr, 0))));
   if (is_concat (u)) return u[1] * unglue (u[2], nr);
   if (is_or (u)) return unglue (u[1], nr) | unglue (u[2], nr);
-  cerr << "\nu= " << u << "\n";
-  cerr << "nr= " << nr << "\n";
+  failed_error << "u = " << u << "\n";
+  failed_error << "nr= " << nr << "\n";
   FAILED ("can't unglue from url");
   return u;
 }
@@ -798,11 +808,13 @@ complete (url base, url sub, url u, string filter, bool flag) {
 url
 complete (url base, url u, string filter, bool flag) {
   // cout << "complete " << base << " |||| " << u << LF;
-  if (is_none (base)) return base;
-  if (is_none (u)) return u;
-  if ((!is_root (base)) && (!is_rooted_name (base))) {
-    cerr << "base= " << base << LF;
-    FAILED ("invalid base url");
+  if (!is_rooted(u)) {
+     if (is_none (base)) return base;
+     if (is_none (u)) return u;
+     if ((!is_root (base)) && (!is_rooted_name (base))) {
+        failed_error << "base= " << base << LF;
+        FAILED ("invalid base url");
+     }
   }
   if (is_name (u) || (is_concat (u) && is_root (u[1]) && is_name (u[2]))) {
     url comp= base * u;
@@ -814,8 +826,8 @@ complete (url base, url u, string filter, bool flag) {
       if (is_of_type (comp, filter)) return u;
       return url_none ();
     }
-    cerr << LF << "base= " << base << LF;
-    cerr << "u= " << u << LF;
+    failed_error << "base= " << base << LF;
+    failed_error << "u= " << u << LF;
     ASSERT (is_rooted (comp), "unrooted url");
     FAILED ("bad protocol in url");
   }
@@ -823,6 +835,27 @@ complete (url base, url u, string filter, bool flag) {
     // FIXME: test filter flags here
     return u;
   }
+  if (is_concat (u) && is_wildcard (u[1], 0) && is_wildcard (u[2], 1)) {
+    // FIXME: ret= ret | ... is unefficient (quadratic) in main loop
+    if (!(is_rooted (base, "default") || is_rooted (base, "file"))) {
+      failed_error << "base= " << base << LF;
+      FAILED ("wildcards only implemented for files");
+    }
+    url ret= url_none ();
+    bool error_flag;
+    array<string> dir= read_directory (base, error_flag);
+    int i, n= N(dir);
+    for (i=0; i<n; i++) {
+      if ((!is_none (ret)) && flag) return ret;
+      if ((dir[i] == ".") || (dir[i] == "..")) continue;
+      if (starts (dir[i], "http://") || starts (dir[i], "ftp://"))
+        if (is_directory (base * dir[i])) continue;
+      ret= ret | (dir[i] * complete (base * dir[i], u, filter, flag));
+      if (match_wildcard (dir[i], u[2][1]->t->label))
+	ret= ret | complete (base, dir[i], filter, flag);
+    }
+    return ret;
+  }
   if (is_concat (u)) {
     url sub= complete (base, u[1], "", false);
     // "" should often be faster than the more correct "d" here
@@ -836,7 +869,7 @@ complete (url base, url u, string filter, bool flag) {
   if (is_wildcard (u)) {
     // FIXME: ret= ret | ... is unefficient (quadratic) in main loop
     if (!(is_rooted (base, "default") || is_rooted (base, "file"))) {
-      cerr << LF << "base= " << base << LF;
+      failed_error << "base= " << base << LF;
       FAILED ("wildcards only implemented for files");
     }
     url ret= url_none ();
@@ -847,6 +880,8 @@ complete (url base, url u, string filter, bool flag) {
     for (i=0; i<n; i++) {
       if ((!is_none (ret)) && flag) return ret;
       if ((dir[i] == ".") || (dir[i] == "..")) continue;
+      if (starts (dir[i], "http://") || starts (dir[i], "ftp://"))
+        if (is_directory (base * dir[i])) continue;
       if (is_wildcard (u, 0))
 	ret= ret | (dir[i] * complete (base * dir[i], u, filter, flag));
       else if (match_wildcard (dir[i], u[1]->t->label))
@@ -854,7 +889,7 @@ complete (url base, url u, string filter, bool flag) {
     }
     return ret;
   }
-  cout << LF << "url= " << u << LF;
+  failed_error << "url= " << u << LF;
   FAILED ("bad url");
   return u;
 }
@@ -965,8 +1000,8 @@ concretize (url u) {
   if (is_here (u)) return as_string (url_pwd ());
   if (is_parent (u)) return as_string (url_pwd () * url_parent ());
   if (is_wildcard (u, 1)) return u->t[1]->label;
-  cerr << "TeXmacs] couldn't concretize " << u->t << LF;
-  // cerr << "\nu= " << u << LF;
+  std_warning << "Couldn't concretize " << u->t << LF;
+  // failed_error << "u= " << u << LF;
   // FAILED ("url has no root");
   return "xxx";
 }
@@ -976,7 +1011,7 @@ materialize (url u, string filter) {
   // Combines resolve and concretize
   url r= resolve (u, filter);
   if (!(is_rooted (r) || is_here (r) || is_parent (r))) {
-    cerr << "\nu= " << u << LF;
+    failed_error << "u= " << u << LF;
     FAILED ("url could not be resolved");
   }
   return concretize (r);
