@@ -13,7 +13,10 @@
 #include <locale.h>
 #endif
 
+#if !defined(__MINGW__) && !defined(__MINGW32__)
 #include <langinfo.h>
+#endif
+
 #include "analyze.hpp"
 #include "hyphenate.hpp"
 #include "impl_language.hpp"
@@ -198,6 +201,7 @@ ucs_text_language_rep::hyphenate (
 ******************************************************************************/
 
 struct oriental_language_rep: language_rep {
+  hashmap<string,bool> punct;
   oriental_language_rep (string lan_name);
   text_property advance (tree t, int& pos);
   array<int> get_hyphens (string s);
@@ -205,43 +209,91 @@ struct oriental_language_rep: language_rep {
 };
 
 oriental_language_rep::oriental_language_rep (string lan_name):
-  language_rep (lan_name) {}
+  language_rep (lan_name), punct (false)
+{
+  punct (".")= true;
+  punct (",")= true;
+  punct (":")= true;
+  punct (";")= true;
+  punct ("!")= true;
+  punct ("?")= true;
+  punct ("<#3000>")= true;
+  punct ("<#3001>")= true;
+  punct ("<#3002>")= true;
+  punct ("<#3003>")= true;
+  punct ("<#3004>")= true;
+  punct ("<#3005>")= true;
+  punct ("<#3006>")= true;
+  punct ("<#3007>")= true;
+  punct ("<#3008>")= true;
+  punct ("<#3009>")= true;
+  punct ("<#300a>")= true;
+  punct ("<#300b>")= true;
+  punct ("<#300c>")= true;
+  punct ("<#300d>")= true;
+  punct ("<#300e>")= true;
+  punct ("<#300f>")= true;
+  punct ("<#300A>")= true;
+  punct ("<#300B>")= true;
+  punct ("<#300C>")= true;
+  punct ("<#300D>")= true;
+  punct ("<#300E>")= true;
+  punct ("<#300F>")= true;
+  punct ("<#ff01>")= true;
+  punct ("<#ff0c>")= true;
+  punct ("<#ff0e>")= true;
+  punct ("<#ff1a>")= true;
+  punct ("<#ff1b>")= true;
+  punct ("<#ff1f>")= true;
+  punct ("<#FF01>")= true;
+  punct ("<#FF0C>")= true;
+  punct ("<#FF0E>")= true;
+  punct ("<#FF1A>")= true;
+  punct ("<#FF1B>")= true;
+  punct ("<#FF1F>")= true;
+}
 
 text_property
 oriental_language_rep::advance (tree t, int& pos) {
   string s= t->label;
   if (pos == N(s)) return &tp_normal_rep;
 
-  if (s[pos]==' ') {
+  if (s[pos] == ' ') {
     pos++;
-    if ((pos == N(s)) || (!is_punctuation (s[pos])))
-      return &tp_space_rep;
-    return &tp_blank_rep;
+    return &tp_space_rep;
   }
 
-  int begin= pos;
-  while (pos<N(s) && s[pos] != ' ') {
-    int start= pos;
-    tm_char_forwards (s, pos);
-    string c= s (start, pos);
-    if (starts (c, "<#300") && N(c) == 7) {
-      if (start > begin) pos= start;
-      break;
-    }
+  if (pos < N(s) && !test (s, pos, "<#")) {
+    while (pos < N(s) && s[pos] != ' ' && !test (s, pos, "<#"))
+      tm_char_forwards (s, pos);
+    return &tp_cjk_no_break_rep;
+  }
+
+  int start= pos;
+  tm_char_forwards (s, pos);
+  string c= s (start, pos);
+  int next= pos;
+  tm_char_forwards (s, next);
+  string x= s (pos, next);
+
+  if (punct->contains (c)) {
+    if (punct->contains (x) || pos == N(s))
+      return &tp_cjk_no_break_period_rep;
+    else return &tp_cjk_period_rep;
+  }
+  else {
+    if (punct->contains (x) || pos == N(s))
+      return &tp_cjk_no_break_rep;
+    else return &tp_cjk_normal_rep;
   }
-  return &tp_normal_rep;
 }
 
 array<int>
 oriental_language_rep::get_hyphens (string s) {
-  int i, n= N(s);
-  array<int> T (n-1);
-  for (i=0; i<n-1; i++)
-    T[i]= HYPH_INVALID;
-  for (i=0, tm_char_forwards (s, i); i<n; tm_char_forwards (s, i))
-    if (s[i] == '<')
-      T[i-1]= 0;
-  return T;
+  int i;
+  array<int> penalty (N(s)+1);
+  for (i=0; i<N(penalty); i++) penalty[i]= HYPH_INVALID;
+  return penalty;
 }
 
 void
@@ -267,6 +319,7 @@ windows_locale_to_language (string s) {
     return "taiwanese"; // for windows xp
   if (s == "Chinese (Traditional)_Taiwan.950")
     return "taiwanese"; // for windows 7
+  if (s == "Croatian_Croatia.1250") return "croatian";
   if (s == "Czech_Czech Republic.1250") return "czech";
   if (s == "Danish_Denmark.1252") return "danish";
   if (s == "Dutch_Netherlands.1252") return "dutch";
@@ -305,6 +358,7 @@ locale_to_language (string s) {
   if (s == "fi") return "finnish";
   if (s == "fr") return "french";
   if (s == "de") return "german";
+  if (s == "hr") return "croatian";
   if (s == "hu") return "hungarian";
   if (s == "it") return "italian";
   if (s == "ja") return "japanese";
@@ -326,6 +380,7 @@ language_to_locale (string s) {
   if (s == "british")    return "en_GB";
   if (s == "bulgarian")  return "bg_BG";
   if (s == "chinese")    return "zh_CN";
+  if (s == "croatian")   return "hr_HR";
   if (s == "czech")      return "cs_CZ";
   if (s == "danish")     return "da_DK";
   if (s == "dutch")      return "nl_NL";
@@ -353,6 +408,7 @@ string
 language_to_local_ISO_charset (string s) {
   if (s == "bulgarian")  return "ISO-8859-5";
   if (s == "chinese")    return "";
+  if (s == "croatian")   return "ISO-8859-2";
   if (s == "czech")      return "ISO-8859-2";
   if (s == "hungarian")  return "ISO-8859-2";
   if (s == "japanese")   return "";
@@ -385,7 +441,11 @@ get_locale_language () {
 
 string
 get_locale_charset () {
+#if defined(__MINGW__) || defined(__MINGW32__)
+  return ("UTF-8");
+#else
   return nl_langinfo (CODESET);
+#endif
 }
 
 /******************************************************************************
@@ -485,6 +545,7 @@ text_language (string s) {
   if (s == "british")    return make_text_language (s, "ukenglish");
   if (s == "bulgarian")  return make_ucs_text_language (s, "bulgarian");
   if (s == "chinese")    return make_oriental_language (s);
+  if (s == "croatian")   return make_text_language (s, "croatian");
   if (s == "czech")      return make_text_language (s, "czech");
   if (s == "danish")     return make_text_language (s, "danish");
   if (s == "dutch")      return make_text_language (s, "dutch");
@@ -506,7 +567,7 @@ text_language (string s) {
   if (s == "taiwanese")  return make_oriental_language (s);
   if (s == "ukrainian")  return make_ucs_text_language (s, "ukrainian");
   if (s == "verbatim")   return tm_new<verb_language_rep> ("verbatim");
-  cerr << "\nThe language was " << s << "\n";
+  failed_error << "The language was " << s << "\n";
   FAILED ("unknown language");
   return tm_new<verb_language_rep> ("verbatim");
 }
