@@ -93,7 +93,7 @@ socket_link_rep::start () {
 #ifndef __MINGW32__
   if (alive) return "busy";
   if (DEBUG_AUTO)
-    cout << "TeXmacs] Connecting to '" << host << ":" << port << "'\n";
+    debug_io << "Connecting to '" << host << ":" << port << "'\n";
   
   // getting host
   
@@ -168,10 +168,10 @@ send_all (int s, char *buf, int *len) {
 void
 socket_link_rep::write (string s, int channel) {
   if ((!alive) || (channel != LINK_IN)) return;
-  if (DEBUG_IO) cout << "---> " << debug_io_string (s) << "\n";
+  if (DEBUG_IO) debug_io << "---> " << debug_io_string (s) << "\n";
   int len= N(s);
   if (send_all (io, &(s[0]), &len) == -1) {
-    cerr << "TeXmacs] write to '" << host << ":" << port << "' failed\n";
+    io_error << "Write to '" << host << ":" << port << "' failed\n";
     stop ();
   }
 }
@@ -183,12 +183,13 @@ socket_link_rep::feed (int channel) {
   char tempout[1024];
   int r= recv (io, tempout, 1024, 0);
   if (r <= 0) {
-    if (r == 0) cout << "TeXmacs] '" << host << ":" << port << "' hung up\n";
-    else cerr << "TeXmacs] read failed from '" << host << ":" << port << "'\n";
+    if (r == 0) debug_io << host << ":" << port << "' hung up\n";
+    else io_warning << "TeXmacs] read failed from '" << host
+                    << ":" << port << "'\n";
     stop ();
   }
   else if (r != 0) {
-    if (DEBUG_IO) cout << debug_io_string (string (tempout, r));
+    if (DEBUG_IO) debug_io << debug_io_string (string (tempout, r));
     outbuf << string (tempout, r);
   }
 #endif
@@ -256,7 +257,7 @@ socket_callback (void *obj, void* info) {
   socket_link_rep* con= (socket_link_rep*) obj;
   if (!con->alive) {
     //cout << "con= " << con << ", " << con->alive << ", " << con->io << "\n";
-    system_warning ("invalid callback invocation of deleted socket link");
+    io_warning << "invalid callback invocation of deleted socket link\n";
     return;
   }
   bool busy= true;
