@@ -4,15 +4,13 @@
 * DESCRIPTION: animations
 * COPYRIGHT  : (C) 2005  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Boxes/composite.hpp"
 #include "Boxes/construct.hpp"
-#include "timer.hpp"
 #include "file.hpp"
 
 /******************************************************************************
@@ -71,6 +69,7 @@ anim_constant_box_rep::anim_constant_box_rep (path ip, box b, int length2):
 
 void
 anim_constant_box_rep::pre_display (renderer& ren) {
+  (void) ren;
   if (!started) anim_start_at (texmacs_time ());
   else if (!finished) {
     finished= (texmacs_time () - (started_at+length) >= 0);
@@ -149,9 +148,7 @@ public:
 anim_compose_box_rep::anim_compose_box_rep (path ip, array<box> bs2):
   box_rep (ip), bs (bs2), cum_len (N(bs))
 {
-  if (N(bs) == 0)
-    fatal_error ("Empty animation",
-		 "anim_compose_box_rep::anim_compose_box_rep");
+  ASSERT (N(bs) != 0, "empty animation");
 
   started = false;
   finished= false;
@@ -212,6 +209,7 @@ anim_compose_box_rep::find_tag (string name) {
 
 void
 anim_compose_box_rep::pre_display (renderer& ren) {
+  (void) ren;
   if (!started) anim_start_at (texmacs_time ());
   else if (!finished) {
     time_t now= texmacs_time ();
@@ -278,19 +276,19 @@ anim_compose_box_rep::find_box_path (path p, bool& found) {
 
 path
 anim_compose_box_rep::find_tree_path (path bp) {
-  if (atom (bp)) return box_rep::find_tree_path (bp);
+  if (is_atom (bp)) return box_rep::find_tree_path (bp);
   return bs[current]->find_tree_path (bp->next);
 }
 
 cursor
 anim_compose_box_rep::find_cursor (path bp) {
-  if (atom (bp)) return box_rep::find_cursor (bp);
+  if (is_atom (bp)) return box_rep::find_cursor (bp);
   else return bs[current]->find_cursor (bp->next);
 }
 
 selection
 anim_compose_box_rep::find_selection (path lbp, path rbp) {
-  if (!atom (lbp) && !atom (rbp))
+  if (!is_atom (lbp) && !is_atom (rbp))
     return bs[current]->find_selection (lbp->next, rbp->next);
   else return box_rep::find_selection (lbp, rbp);
 }
@@ -333,6 +331,7 @@ anim_repeat_box_rep::anim_repeat_box_rep (path ip, box b):
 
 void
 anim_repeat_box_rep::pre_display (renderer& ren) {
+  (void) ren;
   if (!started) anim_start_at (texmacs_time ());
   else if (length > 0) {
     time_t now= texmacs_time ();
@@ -511,6 +510,7 @@ struct sound_box_rep: public box_rep {
       system ("play", u, "&");
     started= true; }
   void pre_display (renderer& ren) {
+    (void) ren;
     if (!started) anim_start_at (texmacs_time ()); }
   int  anim_length () { return 0; }
   bool anim_started () { return started; }
@@ -542,15 +542,16 @@ decompose_gif (url u) {
 }
 
 static void
-add_frames (array<box>& a, path ip, url u, int w, int h, int msecs) {
+add_frames (array<box>& v, path ip, url u, int w, int h, int a,
+            int msecs, int px) {
   if (is_none (u)) return;
   else if (is_or (u)) {
-    add_frames (a, ip, u[1], w, h, msecs);
-    add_frames (a, ip, u[2], w, h, msecs);
+    add_frames (v, ip, u[1], w, h, a, msecs, px);
+    add_frames (v, ip, u[2], w, h, a, msecs, px);
   }
   else {
-    box imb= image_box (ip, u, w, h, 0.0, 0.0, 1.0, 1.0);
-    a << anim_constant_box (ip, imb, msecs);
+    box imb= image_box (ip, u, w, h, a, px);
+    v << anim_constant_box (ip, imb, msecs);
   }
 }
 
@@ -560,41 +561,41 @@ add_frames (array<box>& a, path ip, url u, int w, int h, int msecs) {
 
 box
 anim_constant_box (path ip, box b, int len) {
-  return new anim_constant_box_rep (ip, b, len);
+  return tm_new<anim_constant_box_rep> (ip, b, len);
 }
 
 box
 anim_compose_box (path ip, array<box> bs) {
-  return new anim_compose_box_rep (ip, bs);
+  return tm_new<anim_compose_box_rep> (ip, bs);
 }
 
 box
 anim_repeat_box (path ip, box b) {
-  return new anim_repeat_box_rep (ip, b);
+  return tm_new<anim_repeat_box_rep> (ip, b);
 }
 
 box
 anim_translate_box (path ip, box b, int len, SI sx, SI sy, SI ex, SI ey) {
-  return new anim_translate_box_rep (ip, b, len, sx, sy, ex, ey);
+  return tm_new<anim_translate_box_rep> (ip, b, len, sx, sy, ex, ey);
 }
 
 box
 anim_progressive_box (path ip, box b, int len, rectangle r1, rectangle r2) {
-  return new anim_progressive_box_rep (ip, b, len, r1, r2);
+  return tm_new<anim_progressive_box_rep> (ip, b, len, r1, r2);
 }
 
 box
 sound_box (path ip, url u, SI h) {
-  return new sound_box_rep (ip, u, h);
+  return tm_new<sound_box_rep> (ip, u, h);
 }
 
 box
-video_box (path ip, url u, SI w, SI h, int msecs, bool repeat_flag) {
+video_box (path ip, url u, SI w, SI h, int alpha, int ms, bool rep, int px) {
   url frames= decompose_gif (u);
   if (is_none (frames)) return empty_box (ip, 0, 0, w, h);
   array<box> bs;
-  add_frames (bs, decorate (ip), frames, w, h, msecs);
-  box b= anim_compose_box (repeat_flag? decorate (ip): ip, bs);
-  if (repeat_flag) return anim_repeat_box (ip, b);
+  add_frames (bs, decorate (ip), frames, w, h, alpha, ms, px);
+  box b= anim_compose_box (rep? decorate (ip): ip, bs);
+  if (rep) return anim_repeat_box (ip, b);
   else return b;
 }
