@@ -4,10 +4,9 @@
 * DESCRIPTION: Boxes with a list of child boxes
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Boxes/composite.hpp"
@@ -68,7 +67,7 @@ composite_box_rep::position () {
   if (n == 0) {
     x1= y1= x3= y3= 0;
     x2= y2= x4= y4= 0;
-    fatal_error ("empty composite box", "composite_box::position");
+    FAILED ("empty composite box");
   }
   else {
     x1= y1= x3= y3= MAX_SI;
@@ -145,7 +144,7 @@ composite_box_rep::find_tag (string name) {
   int i, n= N(bs);
   for (i=0; i<n; i++) {
     path p= bs[i]->find_tag (name);
-    if (!nil (p)) return p;
+    if (!is_nil (p)) return p;
   }
   return path ();
 }
@@ -160,9 +159,9 @@ composite_box_rep::transform (frame fr) {
   int i;
   array<box> bs;
   for (i= 0; i<subnr(); i++) {
-    if (!nil (subbox (i))) {
+    if (!is_nil (subbox (i))) {
       box sb= subbox (i)->transform (fr);
-      if (!nil (sb)) bs << sb;
+      if (!is_nil (sb)) bs << sb;
     }
   }
   return N (bs)==0?box ():composite_box (ip, bs);
@@ -245,6 +244,7 @@ composite_box_rep::find_box_path (path p, bool& found) {
   if (n == 0) return box_rep::find_box_path (p, found);
 
   int start= n>>1, acc= start, step= (start+1)>>1;
+  bool last= false;
   while (step > 0) {
     path sr= bs[acc]->find_rip ();
     while (!is_accessible (sr)) {
@@ -268,7 +268,8 @@ composite_box_rep::find_box_path (path p, bool& found) {
       start= max (0, start- step);
       acc  = min (acc, start);
     }
-    if (step <= 1) break;
+    if (last) break;
+    if (step <= 1) last= true;
     step= (step+1)>>1;
   }
 
@@ -312,13 +313,13 @@ composite_box_rep::find_box_path (path p, bool& found) {
 
 path
 composite_box_rep::find_tree_path (path bp) {
-  if (atom (bp)) return box_rep::find_tree_path (bp);
+  if (is_atom (bp)) return box_rep::find_tree_path (bp);
   return bs[bp->item]->find_tree_path (bp->next);
 }
 
 cursor
 composite_box_rep::find_cursor (path bp) {
-  if (atom (bp)) return box_rep::find_cursor (bp);
+  if (is_atom (bp)) return box_rep::find_cursor (bp);
   else {
     int i= bp->item;
     cursor cu= bs[i]->find_cursor (bp->next);
@@ -331,7 +332,7 @@ composite_box_rep::find_cursor (path bp) {
 
 selection
 composite_box_rep::find_selection (path lbp, path rbp) {
-  if ((!atom (lbp)) && (!atom (rbp)) && (lbp->item == rbp->item)) {
+  if ((!is_atom (lbp)) && (!is_atom (rbp)) && (lbp->item == rbp->item)) {
     int i= lbp->item;
     selection sel= bs[i]->find_selection (lbp->next, rbp->next);
     return selection (translate (sel->rs, sx(i), sy(i)), sel->start, sel->end);
@@ -344,7 +345,7 @@ composite_box_rep::graphical_select (SI x, SI y, SI dist) {
   gr_selections res;
   if (graphical_distance (x, y) <= dist) {
     int i, n= subnr();
-    for (i=0; i<n; i++)
+    for (i=n-1; i>=0; i--)
       res << bs[i]->graphical_select (x- sx(i), y- sy(i), dist);
   }
   return res;
@@ -355,7 +356,7 @@ composite_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
   gr_selections res;
   if (contains_rectangle (x1, y1, x2, y2)) {
     int i, n= subnr();
-    for (i=0; i<n; i++)
+    for (i=n-1; i>=0; i--)
       res << bs[i]->graphical_select (x1- sx(i), y1- sy(i),
 				      x2- sx(i), y2- sy(i));
   }
@@ -381,16 +382,77 @@ concrete_composite_box_rep::find_child (SI x, SI y, SI delta, bool force) {
   return m;
 }
 
+/******************************************************************************
+* Table box
+******************************************************************************/
+
+struct table_box_rep: public concrete_composite_box_rep {
+  int rows, cols;
+  array<SI> x, y;
+  array<string> halign;
+  table_box_rep (
+    path ip, array<box> bs, array<SI> x2, array<SI> y2,
+    array<string> halign2, int cols2):
+      concrete_composite_box_rep (ip, bs, x2, y2, false),
+      rows (N(bs) / cols2), cols (cols2),
+      x (x2), y (y2), halign (halign2) { finalize (); }
+  operator tree () { return tree ("table"); }
+  box adjust_kerning (int mode, double factor);
+};
+
+box
+table_box_rep::adjust_kerning (int mode, double factor) {
+  int n= N(bs), i, j;
+  array<box> adj (n);
+  array<SI>  cdw (n);
+  for (i=0; i<n; i++) {
+    SI l, r;
+    adj[i]= bs[i]->adjust_kerning (TABLE_CELL, factor);
+    bs[i]->get_cell_extents (l, r);
+    SI w1= r - l;
+    adj[i]->get_cell_extents (l, r);
+    SI w2= r - l;
+    cdw[i]= w2 - w1;
+  }
+  SI dx= 0;
+  array<SI> nx (n);
+  for (j=0; j<cols; j++) {
+    for (i=0; i<rows; i++)
+      nx[i*cols+j]= x[i*cols+j] + dx;
+    SI dw= MINUS_INFINITY;
+    for (i=0; i<rows; i++)
+      dw= max (dw, cdw[i*cols+j]);
+    dx += dw;
+    for (i=0; i<rows; i++) {
+      int k= i*cols + j;
+      string ha= halign[k];
+      SI d= dw - cdw[k];
+      SI cdx= 0;
+      if (ha[0] == 'l') cdx= 0;
+      if (ha[0] == 'r') cdx= d;
+      if (ha[0] == 'c') cdx= d >> 1;
+      adj[k]= adj[k]->adjust_cell_geometry (cdx, 0, dw);
+    }
+  }
+  return table_box (ip, adj, nx, y, halign, cols);
+}
+
 /******************************************************************************
 * User interface
 ******************************************************************************/
 
 box
 composite_box (path ip, array<box> bs, bool bfl) {
-  return new concrete_composite_box_rep (ip, bs, bfl);
+  return tm_new<concrete_composite_box_rep> (ip, bs, bfl);
 }
 
 box
 composite_box (path ip, array<box> bs, array<SI> x, array<SI> y, bool bfl) {
-  return new concrete_composite_box_rep (ip, bs, x, y, bfl);
+  return tm_new<concrete_composite_box_rep> (ip, bs, x, y, bfl);
+}
+
+box
+table_box (path ip, array<box> bs, array<SI> x, array<SI> y,
+           array<string> halign, int cols) {
+  return tm_new<table_box_rep> (ip, bs, x, y, halign, cols);
 }
