@@ -85,11 +85,14 @@ edit_env_rep::rewrite (tree t) {
       if (N(t)>=5) end  = as_int (exec (t[4]));
       int i, n= max (0, end-start);
       tree r (make_tree_label (t[1]->label), n);
-      for (i=0; i<n; i++)
-	r[i]= tree (make_tree_label (t[0]->label),
-		    tree (ARG, copy (t[2]), as_string (start+i)),
-		    as_string (start+i));
-
+      if (t[0]->label == "identity")
+	for (i=0; i<n; i++)
+	  r[i]= tree (ARG, copy (t[2]), as_string (start+i));
+      else
+	for (i=0; i<n; i++)
+	  r[i]= tree (make_tree_label (t[0]->label),
+		      tree (ARG, copy (t[2]), as_string (start+i)),
+		      as_string (start+i));
       macro_arg= old_var;
       macro_src= old_src;
       return r;
@@ -418,6 +421,8 @@ edit_env_rep::exec (tree t) {
     return exec_hard_id (t);
   case SCRIPT:
     return exec_script (t);
+  case FIND_ACCESSIBLE:
+    return exec_find_accessible (t);
   case HLINK:
   case ACTION:
     return exec_compound (t);
@@ -500,7 +505,10 @@ edit_env_rep::exec_assign (tree t) {
   tree r= exec (t[0]);
   if (is_compound (r)) return tree (ERROR, "bad assign");
   assign (r->label, copy (t[1]));
-  return tree (ASSIGN, r, tree (QUOTE, read (r->label)));
+  tree v= read (r->label);
+  if (is_atomic (v) || is_func (v, MACRO));
+  else v= tree (QUOTE, v);
+  return tree (ASSIGN, r, v);
 }
 
 tree
@@ -602,6 +610,9 @@ edit_env_rep::exec_drd_props (tree t) {
 	if (is_tuple (val, "repeat", 2))
 	  drd->set_arity (l, as_int (val [1]), as_int (val [2]),
 			  ARITY_REPEAT, CHILD_BIFORM);
+	else if (is_tuple (val, "repeat*", 2))
+	  drd->set_arity (l, as_int (val [1]), as_int (val [2]),
+			  ARITY_VAR_REPEAT, CHILD_BIFORM);
 	else if (is_tuple (val, "options", 2))
 	  drd->set_arity (l, as_int (val [1]), as_int (val [2]),
 			  ARITY_OPTIONS, CHILD_BIFORM);
@@ -680,6 +691,20 @@ edit_env_rep::exec_drd_props (tree t) {
 	drd->set_type (l, drd_encode_type (as_string (val)));
 	drd->freeze_type (l);
       }
+      else if (prop == "parameter" &&
+               drd_encode_type (as_string (val)) >= 0) {
+        drd->set_var_type (l, VAR_PARAMETER);
+	drd->set_type (l, drd_encode_type (as_string (val)));
+	drd->freeze_var_type (l);
+	drd->freeze_type (l);
+      }
+      else if (prop == "macro-parameter" &&
+               drd_encode_type (as_string (val)) >= 0) {
+        drd->set_var_type (l, VAR_MACRO_PARAMETER);
+	drd->set_type (l, drd_encode_type (as_string (val)));
+	drd->freeze_var_type (l);
+	drd->freeze_type (l);
+      }
       else if (drd_encode_type (prop) >= 0) {
 	int tp= drd_encode_type (prop);
 	if (is_int (val)) {
@@ -1547,9 +1572,19 @@ edit_env_rep::exec_hard_id (tree t) {
 
 tree
 edit_env_rep::exec_script (tree t) {
-  if (N(t) != 1 && N(t) != 2) return tree (ERROR, "bad script");
-  if (N(t) == 1) return tree (SCRIPT, exec (t[0]));
-  else return tree (SCRIPT, exec (t[0]), expand (t[1], true));
+  int i, n= N(t);
+  if (n < 1) return tree (ERROR, "bad script");
+  tree r (t, n);
+  r[0]= exec (t[0]);
+  for (i=1; i<n; i++)
+    r[i]= exec (t[i]);
+  return r;
+}
+
+tree
+edit_env_rep::exec_find_accessible (tree t) {
+  if (N(t) < 1) return tree (ERROR, "bad find-accessible");
+  return expand (t[0], true);
 }
 
 tree
@@ -1607,8 +1642,9 @@ edit_env_rep::exec_set_binding (tree t) {
       string old_s= tree_as_string (old_value[0]);
       string new_s= tree_as_string (value);
       if (new_s != old_s && !starts (key, "auto-")) {
-	if (new_s == "") system_warning ("Redefined", key);
-	else system_warning ("Redefined " * key * " as", new_s);
+        redefined << tree (TUPLE, key, new_s);
+	//if (new_s == "") typeset_warning << "Redefined " << key << LF;
+	//else typeset_warning << "Redefined " << key << " as " << new_s << LF;
       }
     }
   }
@@ -1624,8 +1660,10 @@ edit_env_rep::exec_get_binding (tree t) {
   if (type != 0 && type != 1) type= 0;
   if (is_func (value, TUPLE) && (N(value) >= 2)) value= value[type];
   else if (type == 1) value= tree (UNINIT);
-  if (complete && value == tree (UNINIT))
-    system_warning ("Undefined reference", key);
+  if (complete && value == tree (UNINIT)) {
+    missing (key)= tree (GET_BINDING, key);
+    //typeset_warning << "Undefined reference " << key << LF;
+  }
   //cout << t << ": " << key << " -> " << value << "\n";
   return value;
 }
@@ -1633,6 +1671,7 @@ edit_env_rep::exec_get_binding (tree t) {
 tree
 edit_env_rep::exec_pattern (tree t) {
   if (N(t)<1) return tree (ERROR, "bad pattern");
+  if (no_patterns && N(t) == 4) return exec (t[3]);
   url im= exec_string (t[0]);
   url image= resolve (relative (base_file_name, im));
   if (is_none (image))
@@ -1671,8 +1710,8 @@ edit_env_rep::exec_pattern (tree t) {
     w= as_string (imw);
     h= as_string (imh);
   }
-  else if ((!is_int (w) && !is_percentage (w)) ||
-	   (!is_int (h) && !is_percentage (h)))
+  else if ((!is_int (w) && !is_percentage (w) && !is_percentage (w, "@")) ||
+	   (!is_int (h) && !is_percentage (h) && !is_percentage (h, "@")))
     return "white";
   tree r (PATTERN, as_string (image), w, h);
   if (N(t) == 4) r << exec (t[3]);
