@@ -12,6 +12,8 @@
 #include "env.hpp"
 #include "page_type.hpp"
 #include "typesetter.hpp"
+#include "Boxes/construct.hpp"
+#include "analyze.hpp"
 
 /******************************************************************************
 * Retrieving the page size
@@ -24,109 +26,115 @@ initialize_default_var_type () {
   if (N(default_var_type) != 0) return;
   hashmap<string,int>& var_type= default_var_type;
 
-  var_type (DPI)               = Env_Fixed;
-  var_type (ZOOM_FACTOR)       = Env_Zoom;
-  var_type (PREAMBLE)          = Env_Preamble;
-  var_type (SAVE_AUX)          = Env_Fixed;
-  var_type (MODE)              = Env_Mode;
-  var_type (INFO_FLAG)         = Env_Info_Level;
+  var_type (DPI)                = Env_Fixed;
+  var_type (ZOOM_FACTOR)        = Env_Zoom;
+  var_type (PREAMBLE)           = Env_Preamble;
+  var_type (SAVE_AUX)           = Env_Fixed;
+  var_type (MODE)               = Env_Mode;
+  var_type (INFO_FLAG)          = Env_Info_Level;
 
-  var_type (FONT)              = Env_Font;
-  var_type (FONT_FAMILY)       = Env_Font;
-  var_type (FONT_SERIES)       = Env_Font;
-  var_type (FONT_SHAPE)        = Env_Font;
-  var_type (FONT_SIZE)         = Env_Font_Size;
-  var_type (FONT_BASE_SIZE)    = Env_Font_Size;
-  var_type (MAGNIFICATION)     = Env_Magnification;
-  var_type (MAGNIFY)           = Env_Magnify;
-  var_type (COLOR)             = Env_Color;
-  var_type (OPACITY)           = Env_Color;
-  var_type (LANGUAGE)          = Env_Language;
+  var_type (FONT)               = Env_Font;
+  var_type (FONT_FAMILY)        = Env_Font;
+  var_type (FONT_SERIES)        = Env_Font;
+  var_type (FONT_SHAPE)         = Env_Font;
+  var_type (FONT_SIZE)          = Env_Font_Size;
+  var_type (FONT_BASE_SIZE)     = Env_Font_Size;
+  var_type (MAGNIFICATION)      = Env_Magnification;
+  var_type (MAGNIFY)            = Env_Magnify;
+  var_type (COLOR)              = Env_Color;
+  var_type (OPACITY)            = Env_Color;
+  var_type (NO_PATTERNS)        = Env_Pattern_Mode;
+  var_type (LANGUAGE)           = Env_Language;
 
-  var_type (MATH_LANGUAGE)     = Env_Language;
-  var_type (MATH_FONT)         = Env_Font;
-  var_type (MATH_FONT_FAMILY)  = Env_Font;
-  var_type (MATH_FONT_SERIES)  = Env_Font;
-  var_type (MATH_FONT_SHAPE)   = Env_Font;
-  var_type (MATH_LEVEL)        = Env_Index_Level;
-  var_type (MATH_DISPLAY)      = Env_Display_Style;
-  var_type (MATH_CONDENSED)    = Env_Math_Condensed;
-  var_type (MATH_VPOS)         = Env_Vertical_Pos;
+  var_type (MATH_LANGUAGE)      = Env_Language;
+  var_type (MATH_FONT)          = Env_Font;
+  var_type (MATH_FONT_FAMILY)   = Env_Font;
+  var_type (MATH_FONT_SERIES)   = Env_Font;
+  var_type (MATH_FONT_SHAPE)    = Env_Font;
+  var_type (MATH_LEVEL)         = Env_Index_Level;
+  var_type (MATH_DISPLAY)       = Env_Display_Style;
+  var_type (MATH_CONDENSED)     = Env_Math_Condensed;
+  var_type (MATH_VPOS)          = Env_Vertical_Pos;
 
-  var_type (PROG_LANGUAGE)     = Env_Language;
-  var_type (PROG_FONT)         = Env_Font;
-  var_type (PROG_FONT_FAMILY)  = Env_Font;
-  var_type (PROG_FONT_SERIES)  = Env_Font;
-  var_type (PROG_FONT_SHAPE)   = Env_Font;
+  var_type (PROG_LANGUAGE)      = Env_Language;
+  var_type (PROG_FONT)          = Env_Font;
+  var_type (PROG_FONT_FAMILY)   = Env_Font;
+  var_type (PROG_FONT_SERIES)   = Env_Font;
+  var_type (PROG_FONT_SHAPE)    = Env_Font;
 
-  var_type (PAR_MODE)          = Env_Paragraph;
-  var_type (PAR_HYPHEN)        = Env_Paragraph;
-  var_type (PAR_WIDTH)         = Env_Paragraph;
-  var_type (PAR_LEFT)          = Env_Paragraph;
-  var_type (PAR_RIGHT)         = Env_Paragraph;
-  var_type (PAR_FIRST)         = Env_Paragraph;
-  var_type (PAR_NO_FIRST)      = Env_Paragraph;
-  var_type (PAR_SEP)           = Env_Paragraph;
-  var_type (PAR_HOR_SEP)       = Env_Paragraph;
-  var_type (PAR_VER_SEP)       = Env_Paragraph;
-  var_type (PAR_LINE_SEP)      = Env_Paragraph;
-  var_type (PAR_PAR_SEP)       = Env_Paragraph;
+  var_type (PAR_MODE)           = Env_Paragraph;
+  var_type (PAR_FLEXIBILITY)    = Env_Paragraph;
+  var_type (PAR_HYPHEN)         = Env_Paragraph;
+  var_type (PAR_SPACING)        = Env_Paragraph;
+  var_type (PAR_KERNING_STRETCH)= Env_Paragraph;
+  var_type (PAR_KERNING_MARGIN) = Env_Paragraph;
+  var_type (PAR_HYPHEN)         = Env_Paragraph;
+  var_type (PAR_WIDTH)          = Env_Paragraph;
+  var_type (PAR_LEFT)           = Env_Paragraph;
+  var_type (PAR_RIGHT)          = Env_Paragraph;
+  var_type (PAR_FIRST)          = Env_Paragraph;
+  var_type (PAR_NO_FIRST)       = Env_Paragraph;
+  var_type (PAR_SEP)            = Env_Paragraph;
+  var_type (PAR_HOR_SEP)        = Env_Paragraph;
+  var_type (PAR_VER_SEP)        = Env_Paragraph;
+  var_type (PAR_LINE_SEP)       = Env_Paragraph;
+  var_type (PAR_PAR_SEP)        = Env_Paragraph;
 
-  var_type (PAGE_TYPE)         = Env_Fixed;
-  var_type (PAGE_BREAKING)     = Env_Fixed;
-  var_type (PAGE_FLEXIBILITY)  = Env_Fixed;
-  var_type (PAGE_WIDTH)        = Env_Page_Extents;
-  var_type (PAGE_HEIGHT)       = Env_Page_Extents;
-  var_type (PAGE_WIDTH_MARGIN) = Env_Page;
-  var_type (PAGE_SCREEN_MARGIN)= Env_Page;
-  var_type (PAGE_NR)           = Env_Page;
-  var_type (PAGE_THE_PAGE)     = Env_Page;
-  var_type (PAGE_ODD)          = Env_Page;
-  var_type (PAGE_EVEN)         = Env_Page;
-  var_type (PAGE_RIGHT)        = Env_Page;
-  var_type (PAGE_TOP)          = Env_Page;
-  var_type (PAGE_BOT)          = Env_Page;
-  var_type (PAGE_USER_HEIGHT)  = Env_Page;
-  var_type (PAGE_ODD_SHIFT)    = Env_Page;
-  var_type (PAGE_EVEN_SHIFT)   = Env_Page;
-  var_type (PAGE_SHRINK)       = Env_Page;
-  var_type (PAGE_EXTEND)       = Env_Page;
-  var_type (PAGE_HEAD_SEP)     = Env_Page;
-  var_type (PAGE_FOOT_SEP)     = Env_Page;
-  var_type (PAGE_ODD_HEADER)   = Env_Page;
-  var_type (PAGE_ODD_FOOTER)   = Env_Page;
-  var_type (PAGE_EVEN_HEADER)  = Env_Page;
-  var_type (PAGE_EVEN_FOOTER)  = Env_Page;
-  var_type (PAGE_THIS_HEADER)  = Env_Page;
-  var_type (PAGE_THIS_FOOTER)  = Env_Page;
-  var_type (PAGE_FNOTE_SEP)    = Env_Page;
-  var_type (PAGE_FNOTE_BARLEN) = Env_Page;
-  var_type (PAGE_FLOAT_SEP)    = Env_Page;
-  var_type (PAGE_MNOTE_SEP)    = Env_Page;
-  var_type (PAGE_MNOTE_WIDTH)  = Env_Page;
+  var_type (PAGE_TYPE)          = Env_Fixed;
+  var_type (PAGE_BREAKING)      = Env_Fixed;
+  var_type (PAGE_FLEXIBILITY)   = Env_Fixed;
+  var_type (PAGE_WIDTH)         = Env_Page_Extents;
+  var_type (PAGE_HEIGHT)        = Env_Page_Extents;
+  var_type (PAGE_WIDTH_MARGIN)  = Env_Page;
+  var_type (PAGE_SCREEN_MARGIN) = Env_Page;
+  var_type (PAGE_NR)            = Env_Page;
+  var_type (PAGE_THE_PAGE)      = Env_Page;
+  var_type (PAGE_ODD)           = Env_Page;
+  var_type (PAGE_EVEN)          = Env_Page;
+  var_type (PAGE_RIGHT)         = Env_Page;
+  var_type (PAGE_TOP)           = Env_Page;
+  var_type (PAGE_BOT)           = Env_Page;
+  var_type (PAGE_USER_HEIGHT)   = Env_Page;
+  var_type (PAGE_ODD_SHIFT)     = Env_Page;
+  var_type (PAGE_EVEN_SHIFT)    = Env_Page;
+  var_type (PAGE_SHRINK)        = Env_Page;
+  var_type (PAGE_EXTEND)        = Env_Page;
+  var_type (PAGE_HEAD_SEP)      = Env_Page;
+  var_type (PAGE_FOOT_SEP)      = Env_Page;
+  var_type (PAGE_ODD_HEADER)    = Env_Page;
+  var_type (PAGE_ODD_FOOTER)    = Env_Page;
+  var_type (PAGE_EVEN_HEADER)   = Env_Page;
+  var_type (PAGE_EVEN_FOOTER)   = Env_Page;
+  var_type (PAGE_THIS_HEADER)   = Env_Page;
+  var_type (PAGE_THIS_FOOTER)   = Env_Page;
+  var_type (PAGE_FNOTE_SEP)     = Env_Page;
+  var_type (PAGE_FNOTE_BARLEN)  = Env_Page;
+  var_type (PAGE_FLOAT_SEP)     = Env_Page;
+  var_type (PAGE_MNOTE_SEP)     = Env_Page;
+  var_type (PAGE_MNOTE_WIDTH)   = Env_Page;
 
-  var_type (POINT_STYLE)       = Env_Point_Style;
-  var_type (LINE_WIDTH)        = Env_Line_Width;
-  var_type (DASH_STYLE)        = Env_Dash_Style;
-  var_type (DASH_STYLE_UNIT)   = Env_Dash_Style_Unit;
-  var_type (FILL_COLOR)        = Env_Fill_Color;
-  var_type (ARROW_BEGIN)       = Env_Line_Arrows;
-  var_type (ARROW_END)         = Env_Line_Arrows;
-  var_type (ARROW_LENGTH)      = Env_Line_Arrows;
-  var_type (ARROW_HEIGHT)      = Env_Line_Arrows;
-  var_type (TEXT_AT_HALIGN)    = Env_Text_At_Halign;
-  var_type (TEXT_AT_VALIGN)    = Env_Text_At_Valign;
-  var_type (GR_FRAME)          = Env_Frame;
-  var_type (GR_GEOMETRY)       = Env_Geometry;
-  var_type (GR_GRID)           = Env_Grid;
-  var_type (GR_GRID_ASPECT)    = Env_Grid_Aspect;
-  var_type (GR_EDIT_GRID)        = Env_Grid;
-  var_type (GR_EDIT_GRID_ASPECT) = Env_Grid_Aspect;
+  var_type (POINT_STYLE)        = Env_Point_Style;
+  var_type (LINE_WIDTH)         = Env_Line_Width;
+  var_type (DASH_STYLE)         = Env_Dash_Style;
+  var_type (DASH_STYLE_UNIT)    = Env_Dash_Style_Unit;
+  var_type (FILL_COLOR)         = Env_Fill_Color;
+  var_type (ARROW_BEGIN)        = Env_Line_Arrows;
+  var_type (ARROW_END)          = Env_Line_Arrows;
+  var_type (ARROW_LENGTH)       = Env_Line_Arrows;
+  var_type (ARROW_HEIGHT)       = Env_Line_Arrows;
+  var_type (TEXT_AT_HALIGN)     = Env_Text_At_Halign;
+  var_type (TEXT_AT_VALIGN)     = Env_Text_At_Valign;
+  var_type (GR_FRAME)           = Env_Frame;
+  var_type (GR_GEOMETRY)        = Env_Geometry;
+  var_type (GR_GRID)            = Env_Grid;
+  var_type (GR_GRID_ASPECT)     = Env_Grid_Aspect;
+  var_type (GR_EDIT_GRID)       = Env_Grid;
+  var_type (GR_EDIT_GRID_ASPECT)= Env_Grid_Aspect;
 
-  var_type (SRC_STYLE)         = Env_Src_Style;
-  var_type (SRC_SPECIAL)       = Env_Src_Special;
-  var_type (SRC_COMPACT)       = Env_Src_Compact;
-  var_type (SRC_CLOSE)         = Env_Src_Close;
+  var_type (SRC_STYLE)          = Env_Src_Style;
+  var_type (SRC_SPECIAL)        = Env_Src_Special;
+  var_type (SRC_COMPACT)        = Env_Src_Compact;
+  var_type (SRC_CLOSE)          = Env_Src_Close;
 }
 
 /******************************************************************************
@@ -159,8 +167,8 @@ edit_env_rep::update_page_pars () {
     page_user_height  = page_height - page_top_margin - page_bottom_margin;
   }
   else {
-    page_width        = get_page_par (PAGE_WIDTH);
-    page_height       = get_page_par (PAGE_HEIGHT);
+    page_width  = get_page_par (PAGE_WIDTH);
+    page_height = get_page_par (PAGE_HEIGHT);
 
     if (width_flag == "false") {
       page_odd_margin   = get_page_par (PAGE_ODD);
@@ -199,6 +207,15 @@ edit_env_rep::update_page_pars () {
       page_bottom_margin= page_height - page_top_margin - page_user_height;
     }
 
+    if (page_type == "user") {
+      if (get_string (PAGE_EVEN) == "auto" &&
+          get_string (PAGE_ODD ) != "auto")
+        page_even_margin= page_odd_margin;
+      if (get_string (PAGE_ODD ) == "auto" &&
+          get_string (PAGE_EVEN) != "auto")
+        page_odd_margin= page_even_margin;
+    }
+
     if (screen_flag) {
       page_odd_margin   = get_length (PAGE_SCREEN_LEFT);
       page_right_margin = get_length (PAGE_SCREEN_RIGHT);
@@ -243,6 +260,92 @@ edit_env_rep::get_page_pars (SI& w, SI& h, SI& width, SI& height,
   */
 }
 
+/******************************************************************************
+* Retrieving ornament parameters
+******************************************************************************/
+
+static tree
+tuplify (tree t) {
+  array<string> a= tokenize (t->label, ",");
+  tree r (TUPLE, N(a));
+  for (int i=0; i<N(a); i++)
+    r[i]= a[i];
+  return r;
+}
+
+ornament_parameters
+edit_env_rep::get_ornament_parameters () {
+  tree  shape = read (ORNAMENT_SHAPE);
+  tree  tst   = read (ORNAMENT_TITLE_STYLE);
+  tree  bg    = read (ORNAMENT_COLOR);
+  tree  xc    = read (ORNAMENT_EXTRA_COLOR);
+  tree  sunny = read (ORNAMENT_SUNNY_COLOR);
+  tree  shadow= read (ORNAMENT_SHADOW_COLOR);
+  int   a     = alpha;
+  tree  w     = read (ORNAMENT_BORDER);
+  tree  ext   = read (ORNAMENT_SWELL);
+  tree  xpad  = read (ORNAMENT_HPADDING);
+  tree  ypad  = read (ORNAMENT_VPADDING);
+
+  array<brush> border;
+  if (is_func (sunny, TUPLE)) {
+    for (int i=0; i<N(sunny); i++)
+      border << brush (sunny[i], a);
+  }
+  else {
+    border << brush (sunny, a);
+    border << brush (shadow, a);
+  }
+  if (N(border) == 1) border << border[0];
+  if (N(border) == 2) border << border[1] << border[0];
+
+  SI lw, bw, rw, tw;
+  if (is_atomic (w) && occurs (",", w->label))
+    w= tuplify (w);
+  if (is_func (w, TUPLE, 4)) {
+    lw= ((as_length (w[0]) >> 1) << 1);
+    bw= ((as_length (w[1]) >> 1) << 1);
+    rw= ((as_length (w[2]) >> 1) << 1);
+    tw= ((as_length (w[3]) >> 1) << 1);
+  }
+  else lw= bw= rw= tw= ((as_length (w) >> 1) << 1);
+
+  double lx, bx, rx, tx;
+  if (is_atomic (ext) && occurs (",", ext->label))
+    ext= tuplify (ext);
+  if (is_func (ext, TUPLE, 4)) {
+    lx= as_double (ext[0]);
+    bx= as_double (ext[1]);
+    rx= as_double (ext[2]);
+    tx= as_double (ext[3]);
+  }
+  else lx= bx= rx= tx= as_double (ext);
+
+  SI lpad, rpad;
+  if (is_atomic (xpad) && occurs (",", xpad->label))
+    xpad= tuplify (xpad);
+  if (is_func (xpad, TUPLE, 2)) {
+    lpad= as_length (xpad[0]);
+    rpad= as_length (xpad[1]);
+  }
+  else lpad= rpad= as_length (xpad);
+
+  SI bpad, tpad;
+  if (is_atomic (ypad) && occurs (",", ypad->label))
+    ypad= tuplify (ypad);
+  if (is_func (ypad, TUPLE, 2)) {
+    bpad= as_length (ypad[0]);
+    tpad= as_length (ypad[1]);
+  }
+  else bpad= tpad= as_length (ypad);
+
+  return ornament_parameters (shape, tst,
+                              lw, bw, rw, tw,
+			      lx, bx, rx, tx,
+			      lpad, bpad, rpad, tpad,
+                              brush (bg, a), brush (xc, a), border);
+}
+
 /******************************************************************************
 * Updating the environment from the variables
 ******************************************************************************/
@@ -304,6 +407,24 @@ edit_env_rep::update_color () {
   else fill_brush= brush (fc, alpha);
 }
 
+void
+edit_env_rep::update_pattern_mode () {
+  no_patterns= (get_string (NO_PATTERNS) == "true");
+  if (no_patterns) {
+    tree c= env[COLOR];
+    if (is_func (c, PATTERN, 4)) env (COLOR)= exec (c);
+    c= env[BG_COLOR];
+    if (is_func (c, PATTERN, 4)) env (BG_COLOR)= exec (c);
+    c= env[FILL_COLOR];
+    if (is_func (c, PATTERN, 4)) env (FILL_COLOR)= exec (c);
+    c= env[ORNAMENT_COLOR];
+    if (is_func (c, PATTERN, 4)) env (ORNAMENT_COLOR)= exec (c);
+    c= env[ORNAMENT_EXTRA_COLOR];
+    if (is_func (c, PATTERN, 4)) env (ORNAMENT_EXTRA_COLOR)= exec (c);
+    update_color ();
+  }
+}
+
 void
 edit_env_rep::update_mode () {
   string s= get_string (MODE);
@@ -566,6 +687,7 @@ edit_env_rep::update () {
   update_frame ();
   point_style= get_string (POINT_STYLE);
   update_color ();
+  update_pattern_mode ();
   update_dash_style ();
   dash_style_unit= get_length (DASH_STYLE_UNIT);
   update_line_arrows ();
@@ -631,6 +753,9 @@ edit_env_rep::update (string s) {
   case Env_Color:
     update_color ();
     break;
+  case Env_Pattern_Mode:
+    update_pattern_mode ();
+    break;
   case Env_Paragraph:
     break;
   case Env_Page:
